package justfun.common.dbutil;

import java.io.Reader;
import java.sql.Clob;
import java.sql.Connection;
import java.sql.SQLException;

import justfun.common.pool.Pool;
import justfun.common.pool.PoolManager;

/**
 * DB 데이터 처리를 위한 유틸리티
 * 
 * @author 조재호
 */
public class DbUtil {

	protected DbUtil() {
	}

	// #######################################################
	// ##### 데이터 처리
	// #####

	/**
	 * Clob 데이터를 String으로 변환
	 * 
	 * @param clobData
	 * @return
	 * @throws SQLException
	 */
	public static String getString(Clob clobData) throws SQLException {

		if (clobData == null)
			return null;

		Reader reader = clobData.getCharacterStream();
		StringBuffer buf = new StringBuffer();

		try {
			char temp[] = new char[1024];
			int res = 0;
			do {
				res = reader.read(temp);
				if (res == -1)
					break;
				buf.append(temp, 0, res);
			}
			while (true);
		}
		catch (Exception e) {
			throw new SQLException(e.getMessage());
		}
		finally {
			try {
				reader.close();
				clobData = null;
			}
			catch (Exception e) {
			}
		}

		return buf.toString();
	}

	// #######################################################
	// ##### 풀 사용
	// #####

	/**
	 * 해당 name의 풀로부터 커넥션을 얻는다.
	 * 
	 * @param pool_name
	 * @return Connection
	 */
	public static Connection requestConnection(String pool_name) {
		PoolManager pmgr = PoolManager.getInstance();
		Pool pool = pmgr.requestPool(pool_name);
		Connection conn = (Connection) pool.requestObject();

		return conn;
	}

	/**
	 * 해당 name의 풀로 커넥션을 반환한다.
	 * 
	 * @param pool_name
	 * @param conn
	 */
	public static void returnConnection(String pool_name, Connection conn) {
		if (conn != null) {
			PoolManager pmgr = PoolManager.getInstance();
			Pool pool = pmgr.requestPool(pool_name);
			pool.returnObject(conn);
		}
	}

}
