package justfun.common.dbutil;

import java.sql.CallableStatement;
import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Types;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.dbutils.DbUtils;
import org.apache.commons.dbutils.QueryRunner;

/**
 * QueryRunner 유틸리티
 * 
 * @author 조재호
 */
public class QueryRunnerUtil extends QueryRunner {

	public QueryRunnerUtil() {
		super();
	}

	protected void setAutoCommit(Connection conn, boolean autoCommit) throws SQLException {
		if (conn != null) {
			conn.setAutoCommit(autoCommit);
		}
	}

	/**
	 * 프로시져 호출
	 * 
	 * @param conn
	 * @param query
	 * @param params
	 */
	public void callProcedure(Connection conn, String query) throws SQLException {
		CallableStatement stmt = null;
		try {
			stmt = conn.prepareCall(query);
			stmt.execute();
		}
		catch (SQLException e) {
			this.rethrow(e, query, null);
		}
		finally {
			DbUtils.close(stmt);
		}
	}

	/**
	 * 프로시져 호출
	 * 
	 * @param conn
	 * @param query
	 * @param params
	 */
	public void callProcedure(Connection conn, String query, Object[] params) throws SQLException {
		CallableStatement stmt = null;
		try {
			stmt = conn.prepareCall(query);
			for (int i = 0; params != null && i < params.length; i++) {
				stmt.setObject(i + 1, params[i]);
			}
			stmt.execute();
		}
		catch (SQLException e) {
			this.rethrow(e, query, params);
		}
		finally {
			DbUtils.close(stmt);
		}
	}

	/**
	 * 프로시져 호출
	 * 
	 * @param conn
	 * @param query
	 * @param sqlType
	 * @return
	 * @throws SQLException
	 */
	public List callProcedure(Connection conn, String query, int[] sqlType) throws SQLException {
		List output = null;

		CallableStatement stmt = null;
		try {
			stmt = conn.prepareCall(query);

			for (int i = 0; sqlType != null && i < sqlType.length; i++) {
				stmt.registerOutParameter(i + 1, sqlType[i]);
			}

			stmt.execute();

			output = new ArrayList();
			for (int i = 0; sqlType != null && i < sqlType.length; i++) {
				output.add(stmt.getObject(i + 1));
			}

		}
		catch (SQLException e) {
			this.rethrow(e, query, null);
		}
		finally {
			DbUtils.close(stmt);
		}

		return output;
	}

	/**
	 * 프로시져 호출
	 * 
	 * @param conn
	 * @param query
	 * @param param
	 * @param sqlType
	 * @return
	 * @throws SQLException
	 */
	public List callProcedure(Connection conn, String query, Object[] param, int[] sqlType)
			throws SQLException {
		List output = null;

		CallableStatement stmt = null;
		try {
			stmt = conn.prepareCall(query);

			int index = 0;
			for (int i = 0; param != null && i < param.length; i++) {
				stmt.setObject(i + 1, param[i]);
				index = i + 1;
			}
			for (int i = 0; sqlType != null && i < sqlType.length; i++) {
				stmt.registerOutParameter(index + i + 1, sqlType[i]);
			}

			stmt.execute();

			output = new ArrayList();
			for (int i = 0; sqlType != null && i < sqlType.length; i++) {
				output.add(stmt.getObject(index + i + 1));
			}

		}
		catch (SQLException e) {
			this.rethrow(e, query, param);
		}
		finally {
			DbUtils.close(stmt);
		}

		return output;
	}

	/**
	 * 함수 호출, "? =" 으로 시작 하는 쿼리를 처리 한다.
	 * 
	 * @param conn
	 * @param query
	 * @return The String returned by the Function.
	 */
	public String callFunction(Connection conn, String query) throws SQLException {
		CallableStatement stmt = null;
		String ret = "";
		try {
			stmt = conn.prepareCall(query);

			stmt.registerOutParameter(1, Types.VARCHAR);
			stmt.execute();

			ret = stmt.getString(1);
		}
		catch (SQLException e) {
			this.rethrow(e, query, null);
		}
		finally {
			DbUtils.close(stmt);
		}

		return ret;
	}

	/**
	 * 함수 호출, "? =" 으로 시작 하는 쿼리를 처리 한다.
	 * 
	 * @param conn
	 * @param query
	 * @param params
	 * @return The String returned by the Function.
	 */
	public String callFunction(Connection conn, String query, Object[] params) throws SQLException {
		CallableStatement stmt = null;
		String ret = "";
		try {
			stmt = conn.prepareCall(query);
			for (int i = 0; params != null && i < params.length; i++) {
				stmt.setObject(i + 2, params[i]);
			}

			stmt.registerOutParameter(1, Types.VARCHAR);
			stmt.execute();

			ret = stmt.getString(1);
		}
		catch (SQLException e) {
			this.rethrow(e, query, params);
		}
		finally {
			DbUtils.close(stmt);
		}

		return ret;
	}

}
