package justfun.common.pool;

import java.util.HashMap;
import java.util.Vector;

/**
 * Pool의 추상 구현
 * 
 * @author 조재호
 * @deprecated
 */
public abstract class BasePool implements Pool {
	protected HashMap pMap;
	private Vector poolObj;
	private Vector used;
	private Vector returnTime;
	String poolname;

	public BasePool() {
		pMap = new HashMap();
		poolObj = new Vector();
		used = new Vector();
		returnTime = new Vector();
	}

	protected void init(String poolname, String driver, String conntimeout, String reqtimeout, String initconn, String minconn, String maxconn) {
		this.poolname = poolname;
		pMap.put("poolname", poolname);
		pMap.put("driver", driver);
		pMap.put("conntimeout", new Long(conntimeout));
		pMap.put("reqtimeout", new Long(reqtimeout));
		pMap.put("initconn", new Integer(initconn));
		pMap.put("minconn", new Integer(minconn));
		pMap.put("maxconn", new Integer(maxconn));
		poolObj = new Vector();
		used = new Vector();
		returnTime = new Vector();
		int initConn = ((Integer) pMap.get("initconn")).intValue();

		for (int j = 0; j < initConn; j++)
			try {
				poolObj.add(createPoolObject());
				used.add("available");
				returnTime.add(new Long(System.currentTimeMillis()));
			}
			catch (Exception e) {
				System.out.println(e.getMessage());
			}
	}

	public Object requestObject() throws PoolException {
		return requestObj();
	}

	public synchronized Object requestObj() throws PoolException {
		long reqtimeout = ((Long) pMap.get("reqtimeout")).longValue() * 1000L;
		int max = ((Integer) pMap.get("maxconn")).intValue();
		for (long reqtime = System.currentTimeMillis(); System.currentTimeMillis() - reqtime < reqtimeout;) {
			try {
				for (int i = 0; i < used.size(); i++)
					if (used.get(i).toString().equals("available"))
						if (poolObj.get(i) == null || !isPoolObjValid(poolObj.get(i))) {
							poolObj.remove(i);
							used.remove(i);
							returnTime.remove(i);
						}
						else {
							used.set(i, "using");
							returnTime.set(i, new Long(System.currentTimeMillis()));
							systemLog("request obj");
							return poolObj.get(i);
						}

			}
			catch (Exception e) {
				throw new PoolException(e.getMessage());
			}
			try {
				if (poolObj.size() >= max) {
					String msg = "limit of connection over |" + poolObj.size();
					msg = msg + "|" + pMap.get("poolname");
					msg = msg + "|" + pMap.get("driver");
					msg = msg + "|" + pMap.get("conntimeout");
					msg = msg + "|" + pMap.get("reqtimeout");
					msg = msg + "|" + pMap.get("minconn");
					msg = msg + "|" + pMap.get("maxconn");
					throw new PoolException(msg);
				}
				poolObj.add(createPoolObject());
				used.add("available");
				returnTime.add(new Long(System.currentTimeMillis()));
			}
			catch (Exception e) {
				throw new PoolException(e);
			}
		}

		throw new PoolException("Request Time-out." + reqtimeout + "|" + poolObj.size() + "|" + (String) pMap.get("driver"));
	}

	public void returnObject(Object obj) throws PoolException {
		returnObj(obj);
	}

	protected synchronized void returnObj(Object obj) throws PoolException {
		int index = poolObj.indexOf(obj);
		if (used.get(index).toString().equals("using")) {
			used.set(index, "available");
			returnTime.set(index, new Long(System.currentTimeMillis()));
			systemLog("return obj");
		}
	}

	public synchronized void manage() {
		// System.out.println("Name ["+pMap.get("poolname")+"] managed ");
		long keeptimeout = ((Long) pMap.get("conntimeout")).longValue() * 1000L;
		long reqtimeout = ((Long) pMap.get("reqtimeout")).longValue() * 1000L;
		int min = ((Integer) pMap.get("minconn")).intValue();

		for (int j = 0; j < used.size(); j++) {

			if (System.currentTimeMillis() - ((Long) returnTime.get(j)).longValue() > reqtimeout)
				try {
					if (used.get(j).equals("using")) {
						poolObjClean(poolObj.get(j));
						poolObj.remove(j);
						used.remove(j);
						returnTime.remove(j);
						systemLog("thread is running");
						j--;
						continue;
					}
				}
				catch (Exception e) {
					System.out.println(e.getMessage());
				}

			if (System.currentTimeMillis() - ((Long) returnTime.get(j)).longValue() > keeptimeout)
				try {
					if (poolObj.size() > min) {
						poolObjClean(poolObj.get(j));
						poolObj.remove(j);
						used.remove(j);
						returnTime.remove(j);
						systemLog("thread is running");
						j--;
					}
				}
				catch (Exception e) {
					System.out.println(e.getMessage());
				}
		}
	}

	protected abstract void poolObjClean(Object obj);

	protected abstract boolean isPoolObjValid(Object obj);

	protected abstract Object createPoolObject() throws Exception;

	public void destroy() {
		for (int j = 0; j < poolObj.size(); j++)
			try {
				poolObjClean(poolObj.get(j));
			}
			catch (Exception exception) {
			}

		poolObj.removeAllElements();
		used.removeAllElements();
		returnTime.removeAllElements();
	}

	protected void systemLog(String msg) {
		int total = used.size();
		int usedConn = 0;
		int remainConn = 0;
		for (int i = 0; i < total; i++)
			if (used.get(i).toString().equals("using"))
				usedConn++;
			else
				remainConn++;

		System.out.println("Connection Status : Name [" + pMap.get("poolname") + "] Total [" + total + "], Using [" + usedConn + "], Remaining [" + remainConn + "], Msg [" + msg + "]");
	}
}
