package justfun.common.pool;

import java.util.Enumeration;
import java.util.Hashtable;

/**
 * Pool을 관리하는 관리자.
 * 
 * @author 조재호
 */
public class PoolManager {

	private Hashtable poolList;

	/**
	 * PoolManager 객체를 얻는다.
	 * 
	 * @return
	 */
	public static PoolManager getInstance() {
		return PoolManagerHolder.INSTANCE;
	}

	private static class PoolManagerHolder {
		static final PoolManager INSTANCE = new PoolManager();
	}

	private PoolManager() throws PoolException {
		poolList = null;
		poolList = new Hashtable();
	}

	/**
	 * PoolManager가 관리할 Pool을 추가
	 * 
	 * @param poolName
	 * @param pool
	 * @throws PoolException
	 */
	public void addPool(String poolName, Pool pool) throws PoolException {
		poolList.put(poolName, pool);
	}

	/**
	 * PoolManager에 관리되고 있는 Pool을 얻는다.
	 * 
	 * @param poolname
	 * @return
	 */
	public Pool requestPool(String poolname) {
		return (Pool) poolList.get(poolname);
	}

	public void destroy() {
		Enumeration e = poolList.elements();
		while (e.hasMoreElements()) {
			((Pool) e.nextElement()).destroy();
		}

		poolList.clear();
	}

}
