package justfun.common.pool.impl;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.util.HashMap;
import java.util.Vector;

import justfun.common.pool.Pool;
import justfun.common.pool.PoolException;

/**
 * 
 * Class Description Here
 * 
 * @author 조재호
 * @deprecated
 */
public class ConnectionPool implements Pool, Runnable {

	private HashMap pMap;
	private Vector conn;
	private Vector used;
	private Vector returnTime;
	private Thread th;
	ConnectionPool pool;
	String poolname;

	private boolean printLog = false;

	public ConnectionPool(DbInfoBean bean) {
		this(bean.getPoolname(), bean.getDriver(), bean.getUrl(), bean.getUser(), bean.getPw(), bean.getConntimeout(), bean.getReqtimeout(), bean.getInitconn(), bean.getMinconn(), bean.getMaxconn(), true);
	}

	public ConnectionPool(String poolname, String driver, String url, String user, String pw, String conntimeout, String reqtimeout, String initconn, String minconn, String maxconn, boolean printLog)
			throws PoolException {
		pMap = new HashMap();
		conn = new Vector();
		used = new Vector();
		returnTime = new Vector();
		pool = null;

		this.printLog = printLog;

		this.poolname = poolname;
		pMap.put("poolname", poolname);
		pMap.put("driver", driver);
		pMap.put("url", url);
		pMap.put("user", user);
		pMap.put("pw", pw);
		pMap.put("conntimeout", new Long(conntimeout));
		pMap.put("reqtimeout", new Long(reqtimeout));
		pMap.put("initconn", new Integer(initconn));
		pMap.put("minconn", new Integer(minconn));
		pMap.put("maxconn", new Integer(maxconn));
		conn = new Vector();
		used = new Vector();
		returnTime = new Vector();
		int initConn = ((Integer) pMap.get("initconn")).intValue();
		try {
			driver = (String) pMap.get("driver");
			Class.forName(driver);
		}
		catch (Exception e) {
			throw new PoolException(e.getMessage());
		}
		for (int j = 0; j < initConn; j++)
			try {
				conn.add(DriverManager.getConnection(url, user, pw));
				used.add("available");
				returnTime.add(new Long(System.currentTimeMillis()));
			}
			catch (Exception e) {
				System.out.println(e.getMessage());
			}

		th = new Thread(this);
		th.start();
	}

	public Object requestObject() throws PoolException {
		return requestConnection();
	}

	public synchronized Connection requestConnection() throws PoolException {
		long reqtimeout = ((Long) pMap.get("reqtimeout")).longValue();
		for (long reqtime = System.currentTimeMillis(); System.currentTimeMillis() - reqtime < reqtimeout * 1000L;) {
			try {
				systemLog(String.valueOf(used.size()));
				for (int i = 0; i < used.size(); i++) {
					if (used.get(i).toString().equals("available")) {
						if (conn.get(i) == null || ((Connection) conn.get(i)).isClosed()) {
							conn.remove(i);
							used.remove(i);
							returnTime.remove(i);
						}
						else {
							used.set(i, "using");
							returnTime.set(i, new Long(System.currentTimeMillis()));
							systemLog("request");
							return (Connection) conn.get(i);
						}
					}
				}

			}
			catch (SQLException e) {
				throw new PoolException(e.getMessage());
			}
			try {
				String driver = (String) pMap.get("driver");
				Class.forName(driver);
			}
			catch (Exception e) {
				throw new PoolException(e.getMessage());
			}
			String url = (String) pMap.get("url");
			String user = (String) pMap.get("user");
			String pw = (String) pMap.get("pw");
			try {
				if (conn.size() >= ((Integer) pMap.get("maxconn")).intValue())
					throw new Exception("limit of connection over");
				systemLog("add connection");
				conn.add(DriverManager.getConnection(url, user, pw));
				used.add("available");
				returnTime.add(new Long(System.currentTimeMillis()));
			}
			catch (Exception e) {
				System.out.println(e.getMessage());
			}
		}

		throw new PoolException("Request Time-out.");
	}

	public void returnObject(Object con) throws PoolException {
		returnConnection((Connection) con);
	}

	public synchronized void returnConnection(Connection con) throws PoolException {
		int conIndex = conn.indexOf(con);
		if (conIndex < 0)
			throw new PoolException("return error");
		if (used.get(conIndex).toString().equals("using")) {
			used.set(conIndex, "available");
			returnTime.set(conIndex, new Long(System.currentTimeMillis()));
			systemLog("return");
		}
	}
	
	public void printLog(String msg) {
		// TODO Auto-generated method stub
		
	}

	public void run() {
		do {
			for (int j = 0; j < used.size(); j++) {
				if (System.currentTimeMillis() - ((Long) returnTime.get(j)).longValue() > ((Long) pMap.get("conntimeout")).longValue() * 1000L) {
					try {
						if (conn.size() > ((Integer) pMap.get("minconn")).intValue()) {
							((Connection) conn.get(j)).close();
							conn.remove(j);
							used.remove(j);
							returnTime.remove(j);
							systemLog("thread is running");
						}
					}
					catch (Exception e) {
						System.out.println(e.getMessage());
					}
				}
			}

			/* 닫혀 있는 커넥션이 있으면 커넥션 풀을 destroy 한다. */
			boolean doDestroy = false;
			for (int i = 0; i < conn.size(); i++) {
				try {
					doDestroy = (((Connection) conn.get(i)).isClosed() || doDestroy);
				}
				catch (SQLException sqle) {

				}
			}
			if (doDestroy) {
				destroy();
			}

			try {
				Thread.sleep(0x1d4c0L);
			}
			catch (Exception exception) {
			}
		}
		while (true);
	}

	private void systemLog(String msg) {
		if (!printLog)
			return;

		int total = used.size();
		int usedConn = 0;
		int remainConn = 0;
		for (int i = 0; i < total; i++)
			if (used.get(i).toString().equals("using"))
				usedConn++;
			else
				remainConn++;

		System.out.print("Connection Status : Total [" + total + "], Using [" + usedConn + "], Remaining [" + remainConn + "]");
		System.out.println(",    Msg : [" + msg + "]");
	}

	public void destroy() {
		for (int j = 0; j < conn.size(); j++)
			try {
				((Connection) conn.get(j)).close();
				systemLog("destroy");
			}
			catch (Exception exception) {
			}

		conn.removeAllElements();
		used.removeAllElements();
		returnTime.removeAllElements();
		th = null;
		pool = null;

		systemLog("destroy complete");
	}
}
