package justfun.common.pool.impl;

import java.sql.Connection;
import java.sql.SQLException;

import justfun.common.pool.EventDrivenPool;
import justfun.common.pool.PoolException;

import org.apache.commons.dbcp.BasicDataSource;

/**
 * Jakarta Dbcp를 사용하는 Pool
 * 
 * @author 조재호
 */
public class DbcpConnectionPool extends EventDrivenPool {

	private BasicDataSource source;

	public DbcpConnectionPool(DbInfoBean bean) {
		source = new BasicDataSource();

		source.setDriverClassName(bean.getDriver());
		source.setUsername(bean.getUser());
		source.setPassword(bean.getPw());
		source.setUrl(bean.getUrl());

		source.setMaxActive(Integer.parseInt(bean.getMaxconn()));
		source.setMaxIdle(Integer.parseInt(bean.getMaxconn()));
		source.setMinIdle(Integer.parseInt(bean.getMinconn()));

		source.setTimeBetweenEvictionRunsMillis(1000);

		System.out.println("[DbcpConnectionPool] Init Basic DataSource  ");
	}

	protected Object doRequestObject() throws PoolException {
		return requestConnection();
	}

	public Connection requestConnection() throws PoolException {
		try {
			Connection conn = source.getConnection();

			return conn;
		}
		catch (SQLException sqle) {
			throw new PoolException(sqle);
		}
	}

	protected void doReturnObject(Object obj) throws PoolException {
		returnConnection((Connection) obj);
	}

	public void returnConnection(Connection conn) throws PoolException {
		try {
			conn.close();
		}
		catch (SQLException sqle) {
			throw new PoolException(sqle);
		}
	}
	
	public void printLog(String msg) {
		System.out.println("Active : " + source.getNumActive() + "\tIdle : " + source.getNumIdle() + "\tmsg : " + msg);
	}

	public void destroy() throws PoolException {
		try {
			source.close();
		}
		catch (Exception ex) {
		}

		System.out.print("[DbcpConnectionPool] Destroy DataSource");
	}

}
