package justfun.common.cache;

/**
 * Cache Data
 * 
 * @author 조재호
 */
public class CacheData {

	/**
	 * 데이터의 key
	 */
	private Object key;
	/**
	 * 데이터 객체
	 */
	private Object data;
	/**
	 * 생성일
	 */
	private long createdTime;
	/**
	 * 최종 사용일
	 */
	private long lastAccessTime;
	/**
	 * 만료일
	 */
	private long expireTime;
	/**
	 * 사용 횟수
	 */
	private long hit;

	protected CacheData(Object key, Object data, long expireTime) {
		this.key = key;
		this.data = data;

		createdTime = System.currentTimeMillis();
		lastAccessTime = createdTime;
		this.expireTime = expireTime;
	}

	/**
	 * 데이터 생성일을 얻는다.
	 * 
	 * @return 데이터 생성일
	 */
	public long getCreatedTime() {
		return createdTime;
	}

	/**
	 * 데이터를 얻는다.
	 * 
	 * @return 캐쉬 데이터
	 */
	public Object getData() {
		lastAccessTime = System.currentTimeMillis();
		hit++;
		return data;
	}

	/**
	 * 데이터 만료일을 얻는다.
	 * 
	 * @return 데이터 만료일
	 */
	public long getExpireTime() {
		return expireTime;
	}

	/**
	 * 데이터의 키를 얻는다.
	 * 
	 * @return 캐쉬 데이터 키
	 */
	public Object getKey() {
		return key;
	}

	/**
	 * 데이터 최종 사용일을 얻는다.
	 * 
	 * @return 데이터 최종 사용일
	 */
	public long getLastAccessTime() {
		return lastAccessTime;
	}

	/**
	 * 데이터 사용 횟수
	 * 
	 * @return 데이터 사용 횟수
	 */
	public long getHitCount() {
		return hit;
	}

	/**
	 * 데이터 만료 처리
	 */
	public void expire() {
		this.expireTime = System.currentTimeMillis();
	}

}
