package justfun.common.cache;

import java.util.Hashtable;
import java.util.Set;

/**
 * <p>
 * Hashtable을 사용한 일반적인 캐쉬. (Timeout 방식)
 * </p>
 * 
 * @author 조재호
 */
public class NormalCache implements Cache {

	// public static final long UNLIMITED = -1;

	/* 데이터 캐싱 - Hashtable은 내부적으로 동기화되므로 명시적으로 동기화할 필요가 없다. */
	protected Hashtable cache;

	public NormalCache() {
		cache = new Hashtable();
	}

	public NormalCache(int initialCapacity) {
		cache = new Hashtable(initialCapacity);
	}

	public Object put(Object key, Object value, long expireTime) {
		CacheData data = new CacheData(key, value, expireTime);
		return cache.put(key, data);
	}

	public Object get(Object key) {
		CacheData data = (CacheData) cache.get(key);

		if (data == null)
			return null;
		/* 만료일이 지난 데이터일 경우, 캐쉬에서 제거 */
		if (data.getExpireTime() != UNLIMITED && System.currentTimeMillis() > data.getExpireTime()) {
			cache.remove(data.getKey());
			return null;
		}

		return data.getData();
	}

	public long getCreatedTime(Object key) {
		CacheData data = (CacheData) cache.get(key);
		if (data == null)
			return 0;

		return data.getCreatedTime();
	}

	public long getLastAccessTime(Object key) {
		CacheData data = (CacheData) cache.get(key);
		if (data == null)
			return 0;

		return data.getLastAccessTime();
	}

	public long getExpireTime(Object key) {
		CacheData data = (CacheData) cache.get(key);
		if (data == null)
			return 0;

		return data.getExpireTime();
	}

	public long getHitCount(Object key) {
		CacheData data = (CacheData) cache.get(key);
		if (data == null)
			return -1;

		return data.getHitCount();
	}

	public Object remove(Object key) {
		return cache.remove(key);
	}

	public void removeAll() {
		Object[] keys = cache.keySet().toArray();
		for (int i = keys.length - 1; i >= 0; i--)
			cache.remove(keys[i]);
	}

	public Object expire(Object key) {
		CacheData data = (CacheData) cache.get(key);
		if (data == null)
			return null;
		data.expire();

		return data;
	}

	public void expireAll() {
		Object[] keys = cache.keySet().toArray();
		for (int i = keys.length - 1; i >= 0; i--)
			expire(keys[i]);
	}

	public boolean containsKey(Object key) {
		Object data = get(key);

		return (data == null) ? false : true;
	}

	public Set keySet() {
		return cache.keySet();
	}

	public int size() {
		return cache.size();
	}

	public String toString() {
		return cache.toString();
	}

}
