package justfun.common.util;

import java.beans.IntrospectionException;
import java.beans.PropertyDescriptor;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.util.Enumeration;

import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;

/**
 * Bean 관련 유틸리티
 * 
 * @author 조재호
 */
public class BeanUtil {

	/**
	 * Request Parameter로부터 얻은 값을 Bean에 setting
	 * 
	 * @param bean
	 * @param request
	 * @return 메소드 수행 성공 시 true, 실패 시 false 리턴
	 */
	public static boolean putParamToBean(Object bean, HttpServletRequest request) {
		Enumeration enumeration = request.getParameterNames();
		while (enumeration.hasMoreElements()) {
			String param = (String) enumeration.nextElement();
			String values[] = request.getParameterValues(param);

			try {
				(new PropertyDescriptor(param, bean.getClass())).getWriteMethod().invoke(bean, values);
			}
			catch (IntrospectionException ie) {
				ie.printStackTrace();
			}
			catch (Exception e) {
				e.printStackTrace();
				return false;
			}
		}
		return true;
	}

	/**
	 * ResultSet으로부터 얻은 값을 Bean에 setting
	 * 
	 * @param bean
	 * @param rs
	 * @return 메소드 수행 성공 시 true, 실패 시 false 리턴
	 */
	public static boolean putRsToBean(Object bean, ResultSet rs) {
		ResultSetMetaData rsmd = null;
		int count = 0;
		try {
			rsmd = rs.getMetaData();
			count = rsmd.getColumnCount();
			String colName = null;
			String values[] = new String[1];
			for (int i = 1; i <= count; i++)
				try {
					colName = rsmd.getColumnName(i).toLowerCase();
					values[0] = rs.getString(i);
					if (values[0] == null)
						values[0] = "";
					(new PropertyDescriptor(colName, bean.getClass())).getWriteMethod().invoke(bean, values);
				}
				catch (IntrospectionException ie) {

				}
				catch (Exception e) {
					colName = rsmd.getColumnName(i).toLowerCase();
					values[0] = "";
					try {
						(new PropertyDescriptor(colName, bean.getClass())).getWriteMethod().invoke(bean, values);
					}
					catch (IntrospectionException ie) {
					}
					catch (Exception de) {
					}
				}

		}
		catch (SQLException sqle) {
			System.out.println("in putRsToBean catch about sqlexception.");
			sqle.printStackTrace();
			return false;
		}
		return true;
	}

	/**
	 * Cookie로부터 얻은 값을 Bean에 setting
	 * 
	 * @param bean
	 * @param request
	 * @return 메소드 수행 성공 시 true, 실패 시 false 리턴
	 */
	public static boolean putCookieToBean(Object bean, HttpServletRequest request) {
		Cookie cooks[] = request.getCookies();
		if (cooks != null) {
			for (int i = 0; i < cooks.length; i++) {
				String values[] = new String[1];
				String param = cooks[i].getName();

				try {
					(new PropertyDescriptor(param, bean.getClass())).getWriteMethod().invoke(bean, values);
				}
				catch (IntrospectionException ie) {
				}
				catch (Exception e) {
					e.printStackTrace();
					return false;
				}
			}

		}
		return true;
	}

}
