package justfun.common.util;

/**
 * 2진수 처리 유틸리티
 * 
 * @author 조재호
 * @version 1.0
 */
public class BinaryUtil {

	// ##########################################
	// Binary Operation
	// ##########################################

	/**
	 * 부호 확장
	 * 
	 * @param s
	 * @param bit
	 * @return
	 */
	public static String signExtension(String s, int bit) {
		return lpad(s, bit, s.charAt(0));
	}

	/**
	 * 십진수를 이진수로 변환
	 * 
	 * @param s
	 * @param length
	 * @return
	 */
	public static String toBinaryString(String s, int length) {
		return lpad(Integer.toBinaryString(Integer.parseInt(s)), length, '0');
	}

	/**
	 * 십진수를 이진수로 변환
	 * 
	 * @param i
	 * @param length
	 * @return
	 */
	public static String toBinaryString(int i, int length) {
		return lpad(Integer.toBinaryString(i), length, '0');
	}

	/**
	 * 16진수를 이진수로 변환
	 * 
	 * @param s
	 * @param length
	 * @return
	 */
	public static String hexToBinaryString(String s, int length) {
		return lpad(Integer.toBinaryString(Integer.parseInt(s, 16)), length, '0');
	}

	/**
	 * 이진수를 십진수로 변환
	 * 
	 * @param binary
	 * @return
	 */
	public static int binaryToSignedDecimal(String binary) {
		if (binary.charAt(0) == '1') {
			// 2의 보수법으로 계산
			String s = Long.toBinaryString(Long.parseLong(binary, 2) - 1);
			s = s.replaceAll("[1]", "?");
			s = s.replaceAll("[0]", "1");
			s = s.replaceAll("[?]", "0");
			return -(Integer.parseInt(s, 2));
		}

		return Integer.parseInt(binary, 2);
	}

	public static int binaryToUnsignedDecimal(String binary) {
		return Integer.parseInt(binary, 2);
	}

	// ##########################################
	// String Operation
	// ##########################################

	/**
	 * Left Padding
	 * 
	 * @param s
	 * @param length
	 * @param paddingChar
	 * @return
	 */
	private static String lpad(String s, int length, char paddingChar) {
		if (s != null) {
			int s_len = s.length();
			if (s_len > length) {
				return s.substring(s.length() - length);
			}
			else {
				StringBuffer sb = new StringBuffer();
				for (int i = (length - s_len); i > 0; i--) {
					sb.append(paddingChar);
				}
				sb.append(s);

				return sb.toString();
			}
		}

		return s;
	}

}
