package justfun.common.util;

import java.util.Vector;

/**
 * 문자열 처리 유틸리티
 * 
 * @author 조재호
 */
public class StringUtil {
	/**
	 * str이 null일 경우 defaultStr을 리턴
	 * 
	 * @param str
	 * @param defaultStr
	 * @return String
	 */
	public static String getString(String str, String defaultStr) {
		return (str == null) ? defaultStr : str;
	}

	/**
	 * obj가 null일 경우 defaultStr을 리턴. obj가 null이 아닌 경우 obj.toString() 이 리턴된다.
	 * 
	 * @param obj
	 * @param defaultStr
	 * @return
	 */
	public static String getString(Object obj, String defaultStr) {
		return (obj == null) ? defaultStr : obj.toString();
	}

	/**
	 * HTML 형식으로 반환
	 * 
	 * @param str
	 * @return 변환된 String. str이 <code>null</code>일 경우 null을 리턴
	 */
	public static String toHTML(String str) {
		if (str == null)
			return null;

		String returnStr = str;
		returnStr = returnStr.replaceAll("&", "&amp;"); // &를 제일 먼저 변환
		returnStr = returnStr.replaceAll(" ", "&nbsp;");
		returnStr = returnStr.replaceAll("\"", "&quot;");
		returnStr = returnStr.replaceAll("<", "&lt;"); // \n 보다 앞에...
		returnStr = returnStr.replaceAll(">", "&gt;"); // \n 보다 앞에...
		returnStr = returnStr.replaceAll("\n", "<br>");
		returnStr = returnStr.replaceAll("\r", "");

		return returnStr;
	}

	/**
	 * TEXT 형식으로 반환
	 * 
	 * @param str
	 * @return 변환된 String. str이 <code>null</code>일 경우 null을 리턴
	 */
	public static String toTEXT(String str) {
		if (str == null)
			return null;

		String returnStr = str;
		returnStr = returnStr.replaceAll("<br>", "\n");
		returnStr = returnStr.replaceAll("&gt;", ">");
		returnStr = returnStr.replaceAll("&lt;", "<");
		returnStr = returnStr.replaceAll("&quot;", "\"");
		returnStr = returnStr.replaceAll("&nbsp;", " ");
		returnStr = returnStr.replaceAll("&amp;", "&");

		return returnStr;
	}

	/**
	 * String Array를 String으로 반환
	 * 
	 * @param value
	 * @param str
	 * @return 변환된 String
	 * @throws value가
	 *             <code>null</code>이면 NullPointerException 발생
	 */
	public static String arrToStr(String[] value, String str) {
		String rtn = "";

		for (int i = 0; i < value.length; i++) {
			rtn = rtn + value[i];

			if (i < value.length - 1)
				rtn = rtn + str;
		}

		return rtn;
	}

	/**
	 * String을 Seperator로 나눠서 String Array로 반환
	 * 
	 * @param source
	 * @param sp
	 * @return 변환된 String[]
	 * @throws source가
	 *             <code>null</code>이면 NullPointerException 발생
	 */
	public static String[] strToArr(String source, String sp) {
		Vector newarray = new Vector();

		int i = 0;
		int inpos = 0;

		String temp = "";
		String[] sarray = null;

		try {
			while (source.length() > 0) {
				inpos = source.indexOf(sp);

				if (inpos < 0) {
					inpos = source.length();
				}

				temp = source.substring(0, inpos);
				newarray.addElement(temp);

				if (inpos != source.length()) {
					source = source.substring((inpos + sp.length()), source.length());
				}
				else {
					source = "";
				}

				i++;
			}

			sarray = new String[i];

			for (int j = 0; j < i; j++) {
				sarray[j] = newarray.elementAt(j).toString();
			}

		}
		catch (Exception e) {

		}

		return sarray;
	}

	/**
	 * 특정 길이 이후로 잘라내고 '...'을 붙임
	 * 
	 * @param str
	 * @param len
	 * @return 변환된 String
	 * @throws str이
	 *             <code>null</code>이면 NullPointerException 발생
	 */
	public static String cutString(String str, int len) {
		return cutString(str, len, "...");
	}

	/**
	 * 특정 길이 이후로 잘라냄
	 * 
	 * @param str
	 * @param len
	 * @param extraStr
	 * @return
	 */
	public static String cutString(String str, int len, String extraStr) {
		return (str.length() > len) ? str.substring(0, len) + ((extraStr != null) ? extraStr : "") : str;
	}

	/**
	 * String을 특정길이로 만듬.. 자릿수를 주어진 String으로 채운다. (Left Padding)
	 * 
	 * @param sourceStr
	 * @param dimension
	 * @param insertStr
	 * @return 변환된 String
	 * @throws sourceStr이
	 *             <code>null</code>이면 NullPointerException 발생
	 * @throws insertStr이
	 *             <code>null</code>이면 NullPointerException 발생
	 */
	public static String lpad(String sourceStr, int dimension, String insertStr) {
		String str = sourceStr;
		int len = str.length();

		if (len < dimension) {
			for (int i = 0; i < dimension - len; i++) {
				str = insertStr + str;
			}
		}
		else if (len > dimension) {
			str = str.substring(0, dimension);
		}
		return str;
	}

	/**
	 * String을 특정길이로 만듬.. 자릿수를 주어진 String으로 채운다. (Right Padding)
	 * 
	 * @param sourceStr
	 * @param dimension
	 * @param insertStr
	 * @return 변환된 String
	 * @throws sourceStr이
	 *             <code>null</code>이면 NullPointerException 발생
	 * @throws insertStr이
	 *             <code>null</code>이면 NullPointerException 발생
	 */
	public static String rpad(String sourceStr, int dimension, String insertStr) {
		String str = sourceStr;
		int len = str.length();

		if (len < dimension) {
			for (int i = 0; i < dimension - len; i++) {
				str = str + insertStr;
			}
		}
		else if (len > dimension) {
			str = str.substring(0, dimension);
		}
		return str;
	}

	/**
	 * 숫자형 Left Padding ('0'으로 채운다.)
	 * 
	 * @param sourceStr
	 * @param dimension
	 * @return 변환된 String
	 * @throws sourceStr이
	 *             <code>null</code>이면 NullPointerException 발생
	 */
	public static String DecimalLPad(String sourceStr, int dimension) {
		return lpad(sourceStr, dimension, "0");
	}

	/**
	 * ASCII 코드 반환
	 * 
	 * @param s
	 * @return 변환된 int[] - 아스키 코드 값
	 * @throws s가
	 *             <code>null</code>이면 NullPointerException 발생
	 */
	public static int[] toASCII(String s) {
		int[] ascii = new int[s.length()];
		char[] chars = s.toCharArray();
		int len = chars.length;
		for (int i = 0; i < len; i++) {
			ascii[i] = chars[i];
		}

		return ascii;
	}

	/**
	 * 문자열의 특정 위치에 해당 문자열을 바인딩 ('?'를 대상으로 바인딩)
	 * 
	 * @param targetStr :
	 *            대상 문자열
	 * @param list :
	 *            대체할 문자열 리스트
	 * @return 바인딩 된 String
	 * @throws targetStr이
	 *             <code>null</code>이면 NullPointerException 발생
	 * @throws list가
	 *             <code>null</code>이면 NullPointerException 발생
	 * @throws targetStr에
	 *             바인딩 될 문자열의 갯수와 바인딩 할 문자열의 갯수가 일치하지 않으면 Exception 발생
	 */
	public static String bind(String targetStr, String[] list) throws Exception {
		return bind(targetStr, list, "?");
	}

	/**
	 * 문자열의 특정 위치에 해당 문자열을 바인딩
	 * 
	 * @param targetStr :
	 *            대상 문자열
	 * @param list :
	 *            대체할 문자열 리스트
	 * @param bindingStr :
	 *            대체될 문자열 (해당 문자열을 찾아 바인딩 할 문자열로 대체)
	 * @return 바인딩 된 String
	 * @throws targetStr이
	 *             <code>null</code>이면 NullPointerException 발생
	 * @throws list가
	 *             <code>null</code>이면 NullPointerException 발생
	 * @throws bindingStr이
	 *             <code>null</code>이면 NullPointerException 발생
	 * @throws targetStr에
	 *             바인딩 될 문자열의 갯수와 바인딩 할 문자열의 갯수가 일치하지 않으면 Exception 발생
	 */
	public static String bind(String targetStr, String[] list, String bindingStr) throws Exception {
		String[] sepStr = targetStr.split("[" + bindingStr + "]");
		int bindingPositionCount = sepStr.length - 1;
		if (bindingPositionCount != list.length)
			throw new Exception("Unmatched Binding Count");

		StringBuffer result = new StringBuffer();
		for (int i = 0; i < bindingPositionCount; i++) {
			result.append(sepStr[i]).append(list[i]);
		}
		result.append(sepStr[bindingPositionCount]);

		return result.toString();
	}

	/**
	 * 문자열의 특정 위치에 해당 문자열을 바인딩 ('?'를 대상으로 바인딩)
	 * 
	 * @param targetStr :
	 *            대상 문자열
	 * @param bindingIndex :
	 *            바인딩 할 위치 (1부터 시작)
	 * @param str :
	 *            대체할 문자열
	 * @return 바인딩 된 String
	 * @throws targetStr이
	 *             <code>null</code>이면 NullPointerException 발생
	 * @throws bindingIndex
	 *             위치가 정확하지 않으면 Exception 발생
	 */
	public static String bind(String targetStr, int bindingIndex, String str) throws Exception {
		return bind(targetStr, bindingIndex, str, "?");
	}

	/**
	 * 문자열의 특정 위치에 해당 문자열을 바인딩
	 * 
	 * @param targetStr :
	 *            대상 문자열
	 * @param bindingIndex :
	 *            바인딩 할 위치 (1부터 시작)
	 * @param str :
	 *            대체할 문자열
	 * @param bindingStr :
	 *            대체될 문자열 (해당 문자열을 찾아 바인딩 할 문자열로 대체)
	 * @return 바인딩 된 String
	 * @throws targetStr이
	 *             <code>null</code>이면 NullPointerException 발생
	 * @throws bindingStr이
	 *             <code>null</code>이면 NullPointerException 발생
	 * @throws bindingIndex
	 *             위치가 정확하지 않으면 Exception 발생
	 */
	public static String bind(String targetStr, int bindingIndex, String str, String bindingStr)
			throws Exception {
		int index = -1;
		for (int i = 0; i < bindingIndex; i++) {
			index = targetStr.indexOf(bindingStr, index + 1);
		}

		StringBuffer result = new StringBuffer();
		if (index < 0) {
			throw new Exception("Invalid Binding Index");
		}
		else {
			result.append(targetStr.substring(0, index));
			result.append(str);
			result.append(targetStr.substring(index + bindingStr.length()));
		}

		return result.toString();
	}

}
