/*
 * IDM 어플리케이션 사용자 세션
 *
 */

package justfun.dao.helper;

import java.util.HashMap;
import java.util.Hashtable;
import java.util.Map;

/**
 *
 * @author jhcho
 */
public class Session {
    
    private boolean isTaskEdited = false;
    
    /* 현재 Open한 DAO 파일 정보 */
    public static final String KEY_FILE_OPENED = "file.opened";
    public static final String KEY_FILE_TYPE = "file.type";
    public static final String KEY_FILE_DATA = "file.data";
    
    /* 현재 연결된 Connection 정보 */
    public static final String KEY_CONNECTIONINFO = "connection.info";

    private static class Holder {
	static final Session INSTANCE = new Session();
    }
    
    private Map sessionData;
    
    private Session(){
	// Hashtable is thread-safe Collection
	this.sessionData = new Hashtable();
        clear();
    }
    
    /**
     * 사용자 세션을 얻는다.
     */
    public static Session currentSession(){
	return Holder.INSTANCE;
    }
    
    /**
     * 세션에 값 등록
     */
    public void setAttribute(Object key, Object value){
	this.sessionData.put(key, value);
    }
    
    /**
     * 세션에 등록된 값 얻기
     */
    public Object getAttribute(Object key){
	return this.sessionData.get(key);
    }
    
    /**
     * 세션에 등록된 값 삭제
     */
    public void removeAttribute(Object key){
	this.sessionData.remove(key);
    }
    
    public String toString(){
	return sessionData.toString();
    }
    
    /**
     * 세션에 등록된 값 초기화
     */
    public void clear(){
	this.sessionData.clear();
	// if Map is not support clear()
	if( !this.sessionData.isEmpty() ){
	    this.sessionData = new HashMap();
	}
        setAttribute(KEY_FILE_DATA, new HashMap());
    }
    
}
