/*
 * DBSchemaTreeRenderer.java
 *
 */

package justfun.dao.helper.renderer;

import justfun.dao.helper.model.DatabaseNodeItem;
import justfun.dao.helper.util.Utils;
import java.awt.Component;
import java.awt.Font;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeCellRenderer;
import justfun.dao.helper.model.ColumnNodeItem;
import justfun.dao.helper.model.TableNodeItem;
import justfun.dao.helper.model.TreeNodeItem;

/**
 *
 * @author jhcho
 */
public class SchemaTreeRenderer extends DefaultTreeCellRenderer {
    
    private Icon catalogIcon;
    private Icon tableIcon;
    private Icon columnIcon;
    private Icon pkColumnIcon;
    
    public SchemaTreeRenderer(){
        this.catalogIcon = new ImageIcon(Utils.getResource("catalog.gif"));
        this.tableIcon = new ImageIcon(Utils.getResource("table.gif"));
        this.columnIcon = new ImageIcon(Utils.getResource("column.gif"));
        this.pkColumnIcon = new ImageIcon(Utils.getResource("column_pk.gif"));
    }
    
    public Component getTreeCellRendererComponent(JTree tree, Object value, boolean sel, boolean expanded, boolean leaf, int row, boolean hasFocus) {
        super.getTreeCellRendererComponent(tree, value, sel, expanded, leaf, row, hasFocus);
        
        DefaultMutableTreeNode node = (DefaultMutableTreeNode) value;
        TreeNodeItem item = (TreeNodeItem) node.getUserObject();
        
        Font font = getFont();
        setFont(font.deriveFont(Font.PLAIN));
        
        if( item instanceof DatabaseNodeItem ){
            // catalog element
            if( ((DatabaseNodeItem) item).isConnected() ){
                Font f = getFont();
                setFont(f.deriveFont(Font.BOLD));
            }
            
            setIcon(catalogIcon);
        } else if( item instanceof TableNodeItem ){
            // table element
            setIcon(tableIcon);
        } else if( item instanceof ColumnNodeItem ){
            // column element
            ColumnNodeItem data = (ColumnNodeItem) item;
            if( data.isPrimaryKey() ){
                setIcon(pkColumnIcon);
            } else{
                setIcon(columnIcon);
            }
        }
        
        return this;
    }
    
    
    
}
