/*
 * 프로퍼티 창
 * 
 * | key | value | + button | - buttn | 으로 구성
 *
 */

package justfun.dao.helper.ui;

import justfun.dao.helper.renderer.JComponentCellEditor;
import justfun.dao.helper.util.Utils;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.List;
import java.util.Vector;
import javax.swing.BorderFactory;
import javax.swing.DefaultCellEditor;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.event.CellEditorListener;
import javax.swing.event.EventListenerList;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.DefaultTableModel;

/**
 *
 * @author  jhcho
 */
public class PropertyPanel extends javax.swing.JPanel {
    
    public static final String CMD_ADD = "ADD";
    public static final String CMD_REMOVE = "REMOVE";
    
    private Icon addIcon, removeIcon;
    
    protected EventListenerList listenerList = new EventListenerList();
    
    /** Creates new form PropertyPanel */
    public PropertyPanel() {
	initComponents();
	
	addIcon = new ImageIcon(Utils.getResource("plus.gif"));
	removeIcon = new ImageIcon(Utils.getResource("minus.gif"));
	
	this.scrollPane.setColumnHeader(null);
	
	this.table.setModel(new PropertyTableModel());
	this.table.setDefaultRenderer(Object.class, new PropertyCellRenderer());
	this.table.getColumnModel().getColumn(1).setCellEditor(new PropertyCellEditor());
	this.table.getColumnModel().getColumn(2).setCellEditor(new JComponentCellEditor());
	this.table.getColumnModel().getColumn(3).setCellEditor(new JComponentCellEditor());
    }
    
    /** 버튼 클릭 이벤트 처리를 위한 리스너 등록 */
    public void addActionListener(ActionListener l) {
	listenerList.add(ActionListener.class, l);
    }
    
    public void removeActionListener(ActionListener l) {
	listenerList.remove(ActionListener.class, l);
    }
    
    public void addCellEditorListener(CellEditorListener l) {
	this.table.getColumnModel().getColumn(1).getCellEditor().addCellEditorListener(l);
    }
    
    public void removeCellEditorListener(CellEditorListener l) {
	this.table.getColumnModel().getColumn(1).getCellEditor().removeCellEditorListener(l);
    }
    
    /**
     * remove all rows
     */
    public void clear(){
	((PropertyTableModel) this.table.getModel()).clear();
    }
    
    /**
     * add row. row data = [ key, value, user object ]
     */
    public void addRow(Object[] rowData){
	if( rowData.length != 3 ){
	    throw new IllegalArgumentException("Invalid row data");
	}
	
	JButton btn = new JButton();
	btn.setIcon(this.addIcon);
	btn.setBorder(BorderFactory.createEmptyBorder());
	btn.setName(rowData[0].toString());
	btn.setActionCommand(CMD_ADD);
	btn.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		// focus
		JButton button = (JButton) e.getSource();
		JTable table = (JTable) button.getParent();
		table.requestFocus();
		table.changeSelection(table.getEditingRow(), table.getEditingColumn(), false, false);
		
		// deligate event
		Object[] listeners = listenerList.getListenerList();
		// Process the listeners last to first, notifying those that are interested in this event
		for (int i = listeners.length-2; i>=0; i-=2) {
		    if (listeners[i]==ActionListener.class) {
			((ActionListener)listeners[i+1]).actionPerformed(e);
		    }
		}
	    }
	});
	
	JButton btn1 = new JButton();
	btn1.setIcon(this.removeIcon);
	btn1.setBorder(BorderFactory.createEmptyBorder());
	btn1.setName(rowData[0].toString());
	btn1.setActionCommand(CMD_REMOVE);
	btn1.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		// focus
		JButton button = (JButton) e.getSource();
		JTable table = (JTable) button.getParent();
		table.requestFocus();
		table.changeSelection(table.getEditingRow(), table.getEditingColumn(), false, false);
		
		// deligate event
		Object[] listeners = listenerList.getListenerList();
		// Process the listeners last to first, notifying those that are interested in this event
		for (int i = listeners.length-2; i>=0; i-=2) {
		    if (listeners[i]==ActionListener.class) {
			((ActionListener)listeners[i+1]).actionPerformed(e);
		    }
		}
	    }
	});
	
	Object[] row = new Object[]{ rowData[0], rowData[1], btn, btn1 };
	
	PropertyTableModel model = (PropertyTableModel) this.table.getModel();
	model.addRow(row, rowData[2]);
    }
    
    public int getEditingRow(){
	return this.table.getEditingRow();
    }
    
    public int getEditingColumn(){
	return this.table.getEditingColumn();
    }
    
    /** 현재 선택된 행 인덱스 */
    public int getSelectedRow(){
	return this.table.getSelectedRow();
    }
    
    /** 현재 선택된 컬럼 인덱스 */
    public int getSelectedColumn(){
	return this.table.getSelectedColumn();
    }
    
    /** 특정 행/열의 값 */
    public Object getValueAt(int row, int column){
	return this.table.getModel().getValueAt(row, column);
    }
    
    /** 특정 행/열의 값을 설정 */
    public void setValueAt(Object value, int row, int column){
	this.table.getModel().setValueAt(value, row, column);
    }
    
    /** 특정 행의 Value 값 */
    public Object getPropertyValueAt(int row){
	return getValueAt(row, 1);
    }
    
    /** 특정 행의 Value 값을 설정 */
    public void setPropertyValueAt(Object value, int row){
	setValueAt(value, row, 1);
    }
    
    /** 특정 행의 UserObject 반환 */
    public Object getUserObjectAt(int row){
	return ((PropertyTableModel) this.table.getModel()).getUserObjectAt(row);
    }
    
    /** 특정 행의 UserObject 설정 */
    public void setUserObjectAt(Object value, int row){
	((PropertyTableModel) this.table.getModel()).setUserObjectAt(value, row);
    }
    
    /** 레이블 추가 */
    public void addLabel(String labelText){
	JLabel label = new JLabel(labelText);
	
	DefaultTableModel model = (DefaultTableModel) this.table.getModel();
	model.addRow(new Object[]{ label });
    }
    
    /** 프로퍼티 창을 상위 컴포넌트 크기에 맞춤 */
    public void fitToParentSize(){
	Dimension parentSize = getParent().getSize();
	setSize(parentSize);
	
	int columnMargin = this.table.getColumnModel().getColumnMargin() * this.table.getColumnCount() + 10;
	int cellWidth = (getWidth() - columnMargin - 20) / 2;
	setColumnWidth(0, cellWidth);
	setColumnWidth(1, cellWidth);
	setColumnWidth(2, 10);
	setColumnWidth(3, 10);
    }
    
    private void setColumnWidth(int columnIndex, int width){
	this.table.getTableHeader().getColumnModel().getColumn(columnIndex).setWidth(width);
	this.table.getTableHeader().getColumnModel().getColumn(columnIndex).setPreferredWidth(width);
	this.table.getColumnModel().getColumn(columnIndex).setWidth(width);
	this.table.getColumnModel().getColumn(columnIndex).setPreferredWidth(width);
    }
    
    class PropertyTableModel extends DefaultTableModel {
	private List userObjectPool = new Vector();
	
	public void addRow(Object[] rowData) {
	    userObjectPool.add(null);
	    super.addRow(rowData);
	}
	
	public void addRow(Object[] rowData, Object userObject) {
	    userObjectPool.add(userObject);
	    super.addRow(rowData);
	}
	
	public void clear() {
	    userObjectPool.clear();
	    setRowCount(0);
	}
	
	public Object getUserObjectAt(int row){
	    return userObjectPool.get(row);
	}
	
	public void setUserObjectAt(Object value, int row){
	    userObjectPool.add(row, value);
	}
	
	public boolean isCellEditable(int row, int column) {
	    return (column == 2 || column == 3);
	}
	
	public int getColumnCount() {
	    return 4;
	}
	
	public String getColumnName(int column) {
	    return "";
	}
	
    }
    
    class PropertyCellRenderer extends DefaultTableCellRenderer {
	public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column) {
	    if( value instanceof JComponent ){
		JComponent comp = (JComponent) value;
		return comp;
	    }
	    
	    super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
	    return this;
	}
    }
    
    class PropertyCellEditor extends DefaultCellEditor {
	public PropertyCellEditor(){
	    super(new JTextField());
	    JTextField tf = (JTextField) getComponent();
	    tf.setBorder(BorderFactory.createEmptyBorder());
	}
	
	public Component getTableCellEditorComponent(JTable table, Object value, boolean isSelected, int row, int column) {
	    JTextField tf = (JTextField) super.getTableCellEditorComponent(table, value, isSelected, row, column);
	    tf.setText(value.toString());
	    return tf;
	}
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {
        scrollPane = new javax.swing.JScrollPane();
        table = new javax.swing.JTable();

        addComponentListener(new java.awt.event.ComponentAdapter() {
            public void componentResized(java.awt.event.ComponentEvent evt) {
                formComponentResized(evt);
            }
        });

        scrollPane.setName("ScrollPane");
        table.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {

            },
            new String [] {

            }
        ));
        table.setAutoResizeMode(javax.swing.JTable.AUTO_RESIZE_OFF);
        table.setName("Table");
        scrollPane.setViewportView(table);

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(scrollPane, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 227, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(scrollPane, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 146, Short.MAX_VALUE)
        );
    }// </editor-fold>//GEN-END:initComponents
    
    private void formComponentResized(java.awt.event.ComponentEvent evt) {//GEN-FIRST:event_formComponentResized
	fitToParentSize();
    }//GEN-LAST:event_formComponentResized
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JScrollPane scrollPane;
    private javax.swing.JTable table;
    // End of variables declaration//GEN-END:variables
    
}
