/*
 * 쿼리 & 스키마 브라우저
 *
 */

package justfun.dao.helper.ui;

import java.awt.Graphics2D;
import justfun.dao.helper.Session;
import justfun.dao.helper.model.ColumnNodeItem;
import justfun.dao.helper.model.DatabaseNodeItem;
import justfun.dao.helper.model.TableNodeItem;
import justfun.dao.helper.renderer.SchemaTreeRenderer;
import justfun.dao.helper.util.Database;
import justfun.dao.helper.util.Logger;
import justfun.dao.helper.util.Utils;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.List;
import java.util.Map;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableColumn;
import javax.swing.table.TableModel;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreePath;
import justfun.dao.helper.model.TreeNodeItem;

/**
 *
 * @author  jhcho
 */
public class QueryBrowserPanel extends javax.swing.JPanel {
    
    private DatabaseNodeItem openedDatabase;
    
    /** Creates new form DBPanel */
    public QueryBrowserPanel() {
        initComponents();
        
        this.schema.setModel(null);
        this.schema.setCellRenderer(new SchemaTreeRenderer());
    }
    
    /**
     * 현재 세션 정보에 저장된 Connection 정보를 사용하여 화면을 갱신한다.
     */
    public void updateModelOnCurrentSession(){
        Session session = Session.currentSession();
        
        Map connectionInfo = (Map) session.getAttribute(Session.KEY_CONNECTIONINFO);
        
        try{
            // connect database and get schema
            loadDatabaseSchema();
            
            // resultset
            this.resultset.setModel(new DefaultTableModel());
            this.resultset.updateUI();
            
            this.openedDatabase = null;
        }catch(Exception ex){
            Logger.write(ex);
            
            String errorMessage = ex.getMessage();
            errorMessage += "\n\n(If you want more information, see error log.)";
            JOptionPane.showMessageDialog(this, errorMessage, "Error while schema loading", JOptionPane.ERROR_MESSAGE);
        }
    }
    
    private void executeQuery(String sql) {
        try{
            Map connectionInfo = (Map) Session.currentSession().getAttribute(Session.KEY_CONNECTIONINFO);
            
            String catalog = (openedDatabase != null)? openedDatabase.getName(): null;
            TableModel model = null;
            if( sql.toUpperCase().startsWith("SELECT") ){
                model = Database.executeQuery(connectionInfo, catalog, sql);
                this.output.setText(model.getRowCount() + " row(s) fetched.");
            }else{
                int rows = Database.executeUpdate(connectionInfo, catalog, sql);
                this.output.setText(rows + " row(s) affected, no resultset returned.");
                model = new DefaultTableModel();
            }
            
            this.resultset.setModel(model);
            
            if( model.getRowCount() > 0 ){
                for( int i = 0; i < model.getColumnCount(); i++ ){
                    TableColumn column = this.resultset.getColumnModel().getColumn(i);
                    String value = model.getValueAt(0, i).toString();
                    int valueWidth = new Double(getFont().getStringBounds(value, ((Graphics2D) getGraphics()).getFontRenderContext()).getWidth()).intValue();
                    int width = Math.max(valueWidth, column.getWidth());
                    column.setPreferredWidth( width );
                }
            }
            
            this.resultset.updateUI();
        }catch(Exception ex){
            Logger.write(ex);
            
            String errorMessage = ex.getMessage();
            errorMessage += "\n\n(If you want more information, see error log.)";
            JOptionPane.showMessageDialog(this, errorMessage, "Error while Query execute", JOptionPane.ERROR_MESSAGE);
        }
    }
    
    private void loadDatabaseSchema() throws Exception {
        Session session = Session.currentSession();
        // connect database and get schema
        Map connectionInfo = (Map) session.getAttribute(Session.KEY_CONNECTIONINFO);
        TreeModel schemaModel = Database.getDatabaseSchema(connectionInfo);
        this.schema.setModel(schemaModel);
        this.schema.updateUI();
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {
        schema_popupMenu = new javax.swing.JPopupMenu();
        m_selectAll = new javax.swing.JMenuItem();
        m_clear = new javax.swing.JMenuItem();
        m_drop = new javax.swing.JMenuItem();
        jSeparator1 = new javax.swing.JSeparator();
        m_refresh = new javax.swing.JMenuItem();
        resultset_popupMenu = new javax.swing.JPopupMenu();
        m_textedit = new javax.swing.JMenuItem();
        jSplitPane1 = new javax.swing.JSplitPane();
        jPanel1 = new javax.swing.JPanel();
        jScrollPane1 = new javax.swing.JScrollPane();
        sql = new javax.swing.JTextArea();
        btn_execute = new javax.swing.JButton();
        jSplitPane2 = new javax.swing.JSplitPane();
        jPanel2 = new javax.swing.JPanel();
        jScrollPane4 = new javax.swing.JScrollPane();
        resultset = new javax.swing.JTable();
        jPanel3 = new javax.swing.JPanel();
        jScrollPane3 = new javax.swing.JScrollPane();
        schema = new javax.swing.JTree();
        btn_reconnect = new javax.swing.JButton();
        output = new javax.swing.JLabel();

        schema_popupMenu.addPopupMenuListener(new javax.swing.event.PopupMenuListener() {
            public void popupMenuCanceled(javax.swing.event.PopupMenuEvent evt) {
            }
            public void popupMenuWillBecomeInvisible(javax.swing.event.PopupMenuEvent evt) {
            }
            public void popupMenuWillBecomeVisible(javax.swing.event.PopupMenuEvent evt) {
                schema_popupMenuPopupMenuWillBecomeVisible(evt);
            }
        });

        m_selectAll.setText("Fetch All Rows");
        m_selectAll.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_selectAllActionPerformed(evt);
            }
        });

        schema_popupMenu.add(m_selectAll);

        m_clear.setText("Delete All Rows");
        m_clear.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_clearActionPerformed(evt);
            }
        });

        schema_popupMenu.add(m_clear);

        m_drop.setText("Drop This");
        m_drop.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_dropActionPerformed(evt);
            }
        });

        schema_popupMenu.add(m_drop);

        schema_popupMenu.add(jSeparator1);

        m_refresh.setText("Refresh");
        m_refresh.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_refreshActionPerformed(evt);
            }
        });

        schema_popupMenu.add(m_refresh);

        m_textedit.setText("View in Text Editor");
        m_textedit.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_texteditActionPerformed(evt);
            }
        });

        resultset_popupMenu.add(m_textedit);

        setName("DBPanel");
        jSplitPane1.setBorder(null);
        jSplitPane1.setDividerLocation(100);
        jSplitPane1.setDividerSize(3);
        jSplitPane1.setOrientation(javax.swing.JSplitPane.VERTICAL_SPLIT);
        jPanel1.setBorder(javax.swing.BorderFactory.createTitledBorder("SQL"));
        sql.setColumns(20);
        sql.setRows(2);
        sql.setTabSize(4);
        sql.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyPressed(java.awt.event.KeyEvent evt) {
                sqlKeyPressed(evt);
            }
        });

        jScrollPane1.setViewportView(sql);

        btn_execute.setText("Execute");
        btn_execute.setToolTipText("Execute query (Ctrl + Enter)");
        btn_execute.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btn_executeActionPerformed(evt);
            }
        });

        org.jdesktop.layout.GroupLayout jPanel1Layout = new org.jdesktop.layout.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(org.jdesktop.layout.GroupLayout.TRAILING, jPanel1Layout.createSequentialGroup()
                .add(jScrollPane1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 583, Short.MAX_VALUE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(btn_execute, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 89, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(btn_execute, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 71, Short.MAX_VALUE)
            .add(org.jdesktop.layout.GroupLayout.TRAILING, jScrollPane1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 71, Short.MAX_VALUE)
        );
        jSplitPane1.setTopComponent(jPanel1);

        jSplitPane2.setBorder(null);
        jSplitPane2.setDividerLocation(400);
        jSplitPane2.setDividerSize(3);
        jSplitPane2.addComponentListener(new java.awt.event.ComponentAdapter() {
            public void componentResized(java.awt.event.ComponentEvent evt) {
                jSplitPane2ComponentResized(evt);
            }
        });

        jPanel2.setBorder(javax.swing.BorderFactory.createTitledBorder("Result"));
        resultset.setComponentPopupMenu(resultset_popupMenu);
        resultset.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {
                {}
            },
            new String [] {

            }
        ));
        resultset.setAutoResizeMode(javax.swing.JTable.AUTO_RESIZE_OFF);
        resultset.setName("ResultSet Table");
        jScrollPane4.setViewportView(resultset);

        org.jdesktop.layout.GroupLayout jPanel2Layout = new org.jdesktop.layout.GroupLayout(jPanel2);
        jPanel2.setLayout(jPanel2Layout);
        jPanel2Layout.setHorizontalGroup(
            jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jScrollPane4, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 384, Short.MAX_VALUE)
        );
        jPanel2Layout.setVerticalGroup(
            jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jScrollPane4, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 410, Short.MAX_VALUE)
        );
        jSplitPane2.setLeftComponent(jPanel2);

        jPanel3.setBorder(javax.swing.BorderFactory.createTitledBorder("Database Schema"));
        schema.setComponentPopupMenu(schema_popupMenu);
        schema.setName("Schema Tree");
        schema.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
                schemaMousePressed(evt);
            }
        });

        jScrollPane3.setViewportView(schema);

        org.jdesktop.layout.GroupLayout jPanel3Layout = new org.jdesktop.layout.GroupLayout(jPanel3);
        jPanel3.setLayout(jPanel3Layout);
        jPanel3Layout.setHorizontalGroup(
            jPanel3Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jScrollPane3, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 276, Short.MAX_VALUE)
        );
        jPanel3Layout.setVerticalGroup(
            jPanel3Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jScrollPane3, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 410, Short.MAX_VALUE)
        );
        jSplitPane2.setRightComponent(jPanel3);

        jSplitPane1.setRightComponent(jSplitPane2);

        btn_reconnect.setText("Connect...");
        btn_reconnect.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btn_reconnectActionPerformed(evt);
            }
        });

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(org.jdesktop.layout.GroupLayout.TRAILING, layout.createSequentialGroup()
                .addContainerGap()
                .add(output, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 390, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, 202, Short.MAX_VALUE)
                .add(btn_reconnect))
            .add(jSplitPane1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 695, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(org.jdesktop.layout.GroupLayout.TRAILING, layout.createSequentialGroup()
                .add(jSplitPane1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 542, Short.MAX_VALUE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(btn_reconnect)
                    .add(output)))
        );
    }// </editor-fold>//GEN-END:initComponents
    
    private void m_texteditActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_texteditActionPerformed
        int row = this.resultset.getSelectedRow();
        int col = this.resultset.getSelectedColumn();
        
        if( row >= 0 && col >= 0 ){
            TextEditDialog dialog = new TextEditDialog((JFrame) getTopLevelAncestor(), true);
            dialog.setText(this.resultset.getValueAt(row, col).toString());
            Utils.setDialogLocationToCenter(dialog);
            dialog.setVisible(true);
        }
    }//GEN-LAST:event_m_texteditActionPerformed
    
    private void m_refreshActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_refreshActionPerformed
        try{
            TreePath selectedPath = this.schema.getSelectionPath();
            if( selectedPath != null ){
                DefaultMutableTreeNode selectedNode = (DefaultMutableTreeNode) selectedPath.getLastPathComponent();
                TreeNodeItem item = (TreeNodeItem) selectedNode.getUserObject();
                
                if( item instanceof DatabaseNodeItem ){
                    Session session = Session.currentSession();
                    Map connectionInfo = (Map) session.getAttribute(Session.KEY_CONNECTIONINFO);
                    Database.attatchTableSchema(connectionInfo, selectedNode);
                    this.schema.updateUI();
                }else if( item instanceof TableNodeItem ){
                    // TODO
                }else if( item instanceof ColumnNodeItem ){
                    // TODO
                }else{
                    loadDatabaseSchema();
                }
            }
        }catch(Exception ex){
            Logger.write(ex);
            
            String errorMessage = ex.getMessage();
            errorMessage += "\n\n(If you want more information, see error log.)";
            JOptionPane.showMessageDialog(this, errorMessage, "Error while Query execute", JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_m_refreshActionPerformed
    
    private void m_dropActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_dropActionPerformed
        if( JOptionPane.showConfirmDialog(this, "Do you want to DROP selected object?", "DROP selected object", JOptionPane.OK_CANCEL_OPTION) == JOptionPane.OK_OPTION ){
            TreePath selectedPath = this.schema.getSelectionPath();
            if( selectedPath != null ){
                try{
                    DefaultMutableTreeNode selectedNode = (DefaultMutableTreeNode) selectedPath.getLastPathComponent();
                    TreeNodeItem item = (TreeNodeItem) selectedNode.getUserObject();
                    
                    if( item instanceof DatabaseNodeItem ){
                        this.sql.setText("drop database " + item.getName());
                    }else if( item instanceof TableNodeItem ){
                        this.sql.setText("drop table " + item.getName());
                    }
                    this.btn_execute.doClick();
                    
                    Session session = Session.currentSession();
                    Map connectionInfo = (Map) session.getAttribute(Session.KEY_CONNECTIONINFO);
                    Database.attatchTableSchema(connectionInfo, selectedNode);
                    this.schema.updateUI();
                }catch(Exception ex){
                    Logger.write(ex);
                    
                    String errorMessage = ex.getMessage();
                    errorMessage += "\n\n(If you want more information, see error log.)";
                    JOptionPane.showMessageDialog(this, errorMessage, "Error while Query execute", JOptionPane.ERROR_MESSAGE);
                }
            }
        }
    }//GEN-LAST:event_m_dropActionPerformed
    
    private void m_clearActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_clearActionPerformed
        if( JOptionPane.showConfirmDialog(this, "Do you want to DELETE all rows of selected table?", "DELETE all rows", JOptionPane.OK_CANCEL_OPTION) == JOptionPane.OK_OPTION ){
            TreePath selectedPath = this.schema.getSelectionPath();
            if( selectedPath != null ){
                DefaultMutableTreeNode selectedNode = (DefaultMutableTreeNode) selectedPath.getLastPathComponent();
                TreeNodeItem item = (TreeNodeItem) selectedNode.getUserObject();
                
                this.sql.setText("delete from " + item.getName());
                this.btn_execute.doClick();
            }
        }
    }//GEN-LAST:event_m_clearActionPerformed
    
    private void m_selectAllActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_selectAllActionPerformed
        TreePath selectedPath = this.schema.getSelectionPath();
        if( selectedPath != null ){
            DefaultMutableTreeNode selectedNode = (DefaultMutableTreeNode) selectedPath.getLastPathComponent();
            TreeNodeItem item = (TreeNodeItem) selectedNode.getUserObject();
            
            this.sql.setText("select * from " + item.getName());
            this.btn_execute.doClick();
        }
    }//GEN-LAST:event_m_selectAllActionPerformed
    
    private void schema_popupMenuPopupMenuWillBecomeVisible(javax.swing.event.PopupMenuEvent evt) {//GEN-FIRST:event_schema_popupMenuPopupMenuWillBecomeVisible
        TreePath selectedPath = this.schema.getSelectionPath();
        if( selectedPath == null ){
            this.m_selectAll.setEnabled(false);
            this.m_clear.setEnabled(false);
            this.m_drop.setEnabled(false);
            this.m_refresh.setEnabled(false);
            return;
        }
        
        DefaultMutableTreeNode selectedNode = (DefaultMutableTreeNode) selectedPath.getLastPathComponent();
        TreeNodeItem item = (TreeNodeItem) selectedNode.getUserObject();
        
        boolean isDatabase = item instanceof DatabaseNodeItem;
        boolean isTable = item instanceof TableNodeItem;
        boolean isColumn = item instanceof ColumnNodeItem;
        
        this.m_selectAll.setEnabled(isTable);
        this.m_clear.setEnabled(isTable);
        this.m_drop.setEnabled(isDatabase || isTable);
        this.m_refresh.setEnabled(!isTable && !isColumn);
    }//GEN-LAST:event_schema_popupMenuPopupMenuWillBecomeVisible
    
    private void schemaMousePressed(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_schemaMousePressed
        int selRow = this.schema.getRowForLocation(evt.getX(), evt.getY());
        TreePath selPath = this.schema.getPathForLocation(evt.getX(), evt.getY());
        if(selRow != -1) {
            if(evt.getClickCount() == 2) {
                DefaultMutableTreeNode selNode = (DefaultMutableTreeNode) selPath.getLastPathComponent();
                TreeNodeItem item = (TreeNodeItem) selNode.getUserObject();
                if( item instanceof DatabaseNodeItem ){
                    Session session = Session.currentSession();
                    Map connectionInfo = (Map) session.getAttribute(Session.KEY_CONNECTIONINFO);
                    
                    try{
                        if( openedDatabase != null ){
                            openedDatabase.setConnected(false);
                        }
                        openedDatabase = (DatabaseNodeItem) item;
                        openedDatabase.setConnected(true);
                        Database.attatchTableSchema(connectionInfo, selNode);
                        this.schema.updateUI();
                        // this.schema.fireTreeExpanded(selPath);
                    }catch(Exception ex){
                        Logger.write(ex);
                        
                        String errorMessage = ex.getMessage();
                        errorMessage += "\n\n(If you want more information, see error log.)";
                        JOptionPane.showMessageDialog(this, errorMessage, "Error while schema loading", JOptionPane.ERROR_MESSAGE);
                    }
                }
            }
        }
    }//GEN-LAST:event_schemaMousePressed
    
    private void btn_reconnectActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btn_reconnectActionPerformed
        JDialog connectionDialog = new ConnectionDialog((MainFrame) getTopLevelAncestor(), true);
        Utils.setDialogLocationToCenter(connectionDialog);
        
        connectionDialog.setVisible(true);
    }//GEN-LAST:event_btn_reconnectActionPerformed
    
    private void sqlKeyPressed(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_sqlKeyPressed
        switch( evt.getKeyCode() ){
            case KeyEvent.VK_ENTER:
                // Ctrl + Enter 시 execute 버튼 클릭 효과
                if( evt.getModifiersEx() == KeyEvent.CTRL_DOWN_MASK )
                    this.btn_execute.doClick();
                break;
        }
    }//GEN-LAST:event_sqlKeyPressed
    
    private void jSplitPane2ComponentResized(java.awt.event.ComponentEvent evt) {//GEN-FIRST:event_jSplitPane2ComponentResized
        int width = getWidth();
        // make divider position (70% of panel's width)
        this.jSplitPane2.setDividerLocation(width * 70 / 100);
    }//GEN-LAST:event_jSplitPane2ComponentResized
    
    private void btn_executeActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btn_executeActionPerformed
        Map connectionInfo = (Map) Session.currentSession().getAttribute(Session.KEY_CONNECTIONINFO);
        
        if( connectionInfo == null ){
            JOptionPane.showMessageDialog(this, "not connected", "Connection Error", JOptionPane.ERROR_MESSAGE);
            return;
        }
        
        String inputSql = this.sql.getText();
        List userDefinedVar = Utils.extractUserDefinedVariables(inputSql);
        
        if( userDefinedVar != null && userDefinedVar.size() != 0 ){
            UserDefinedVarDialog dialog = new UserDefinedVarDialog((JFrame) getTopLevelAncestor(), true);
            dialog.setTag(inputSql);
            dialog.setVariables(userDefinedVar);
            Utils.setDialogLocationToCenter(dialog);
            dialog.setVisible(true);
            
            dialog.addWindowListener(new WindowAdapter() {
                public void windowClosed(WindowEvent e) {
                    UserDefinedVarDialog win = (UserDefinedVarDialog) e.getWindow();
                    Object tag = win.getTag();
                    Map returnVal = win.getValues();
                    if( returnVal != null ){
                        String query = tag.toString();
                        Object[] key = returnVal.keySet().toArray();
                        for( int i = 0; i < key.length; i++ ){
                            query = query.replaceAll("\\&\\[" + key[i].toString() + "\\]", returnVal.get(key[i]).toString());
                        }
                        executeQuery(query);
                    }
                }
            });
        } else{
            executeQuery(inputSql);
        }
    }//GEN-LAST:event_btn_executeActionPerformed
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton btn_execute;
    private javax.swing.JButton btn_reconnect;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JPanel jPanel3;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JScrollPane jScrollPane3;
    private javax.swing.JScrollPane jScrollPane4;
    private javax.swing.JSeparator jSeparator1;
    private javax.swing.JSplitPane jSplitPane1;
    private javax.swing.JSplitPane jSplitPane2;
    private javax.swing.JMenuItem m_clear;
    private javax.swing.JMenuItem m_drop;
    private javax.swing.JMenuItem m_refresh;
    private javax.swing.JMenuItem m_selectAll;
    private javax.swing.JMenuItem m_textedit;
    private javax.swing.JLabel output;
    private javax.swing.JTable resultset;
    private javax.swing.JPopupMenu resultset_popupMenu;
    private javax.swing.JTree schema;
    private javax.swing.JPopupMenu schema_popupMenu;
    private javax.swing.JTextArea sql;
    // End of variables declaration//GEN-END:variables
    
}
