/*
 * conf/config.xml로 부터 얻은 환경 설정 정보를 나타내는 클래스
 *
 */

package justfun.dao.helper.util;

import java.io.File;
import java.io.FileWriter;
import java.io.PrintWriter;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import org.dom4j.Attribute;
import org.dom4j.Element;
import org.dom4j.Document;
import org.dom4j.io.OutputFormat;
import org.dom4j.io.SAXReader;
import org.dom4j.io.XMLWriter;

/**
 *
 * @author jhcho
 */
public class Config {
    
    private static final String CONFIG_DIRECTORY = "conf";
    private static final String CONFIG_FILE = "config.xml";
    
    /**
     * 초기화. conf/config.xml이 없으면 생성한다.
     */
    public static void initialize(){
	File configDir = new File(CONFIG_DIRECTORY);
	if( !configDir.exists() ){
	    configDir.mkdir();
	}
	File configFile = new File(CONFIG_DIRECTORY, CONFIG_FILE);
	if( !configFile.exists() ){
	    try{
		configFile.createNewFile();
		
		PrintWriter writer = new PrintWriter(new FileWriter(configFile));
		writer.println("<?xml version=\"1.0\" encoding=\"UTF-8\" ?>");
		writer.println("<config />");
		writer.close();
	    }catch(Exception ex){
		ex.printStackTrace();
	    }
	}
    }
    
    /**
     * 환경 설정 파일을 읽어 XML Document를 얻는다.
     */
    protected static Document getDocument() throws Exception {
	SAXReader reader = new SAXReader();
	return reader.read(new File(CONFIG_DIRECTORY, CONFIG_FILE));
    }
    
    /**
     * 환경 설정 파일을 저장한다.
     */
    protected static void writeToXml(Document document) throws Exception {
	File destFile = new File(CONFIG_DIRECTORY, CONFIG_FILE);
	XMLWriter writer = null;
	try {
	    OutputFormat format = OutputFormat.createPrettyPrint();
	    writer = new XMLWriter(new FileWriter(destFile), format);
	    writer.write(document);
	} finally {
	    writer.close();
	}
    }
}
