/*
 * conf/config.xml에 정의된 connection 정보
 *
 */

package justfun.dao.helper.util;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import org.dom4j.Document;
import org.dom4j.Element;

/**
 *
 * @author jhcho
 */
public class Connections extends Config {
    
    public static final String[] dbms = new String[]{ "MySQL", "Oracle", "MSSQL" };
    
    /**
     * connection 목록
     */
    public static List list() throws Exception {
	List list = new ArrayList();
	
	Document document = getDocument();
	Element root = document.getRootElement();
	Element connections = root.element("connections");
	if( connections != null ){
	    List elements = connections.elements("connection");
	    if( elements != null ){
		for( int i = 0; i < elements.size(); i++ ){
		    list.add(Utils.elementToMap((Element) elements.get(i)));
		}
	    }
	}
	
	return list;
    }
    
    /**
     * 특정 connection 얻기
     */
    public static Map get(String connectionName) throws Exception {
	Document document = getDocument();
	Element root = document.getRootElement();
	Element connections = root.element("connections");
	if( connections != null ){
	    List elements = connections.elements("connection");
	    if( elements != null ){
		Element element;
		for( int i = 0; i < elements.size(); i++ ){
		    element = (Element) elements.get(i);
		    if( connectionName.equals(element.attributeValue("name")) ){
			return Utils.elementToMap(element);
		    }
		    
		}
	    }
	}
	return null;
    }
    
    /**
     * connection 정보 삭제
     */
    public static void remove(String connectionName) throws Exception {
	Document document = getDocument();
	Element root = document.getRootElement();
	Element connections = root.element("connections");
	if( connections != null ){
	    List elements = connections.elements("connection");
	    if( elements != null ){
		Element element;
		for( int i = 0; i < elements.size(); i++ ){
		    element = (Element) elements.get(i);
		    if( connectionName.equals(element.attributeValue("name")) ){
			connections.remove(element);
		    }
		}
	    }
	}
	
	writeToXml(document);
    }
    
    /**
     * connection 등록
     */
    public static void add(String connectionName, String user, String password, String host, int port, String dbms, String sid) throws Exception {
	// TODO Connection Name Duplication Check
	
	Document document = getDocument();
	Element root = document.getRootElement();
	Element connections = root.element("connections");
	if( connections == null ){
	    connections = root.addElement("connections");
	}
	
	Element connection = connections.addElement("connection");
	connection.addAttribute("name", connectionName);
	connection.addAttribute("user", user);
	connection.addAttribute("password", password);
	connection.addAttribute("host", host);
	connection.addAttribute("port", Integer.toString(port));
	connection.addAttribute("dbms", dbms);
	connection.addAttribute("sid", sid);
	
	connection.addAttribute("driver", getDriverName(dbms));
	connection.addAttribute("url", getURL(dbms, host, port, sid));
	
	writeToXml(document);
    }
    
    /**
     * connection 수정
     */
    public static void update(String originalName, String connectionName, String user, String password, String host, int port, String dbms, String sid) throws Exception {
	// TODO Connection Name Duplication Check
	
	Document document = getDocument();
	Element root = document.getRootElement();
	Element connections = root.element("connections");
	if( connections != null ){
	    List elements = connections.elements("connection");
	    if( elements != null ){
		Element element;
		for( int i = 0; i < elements.size(); i++ ){
		    element = (Element) elements.get(i);
		    
		    if( originalName.equals(element.attributeValue("name")) ){
			element.addAttribute("name", connectionName);
			element.addAttribute("user", user);
			element.addAttribute("password", password);
			element.addAttribute("host", host);
			element.addAttribute("port", Integer.toString(port));
			element.addAttribute("dbms", dbms);
			element.addAttribute("sid", sid);
			
			element.addAttribute("driver", getDriverName(dbms));
			element.addAttribute("url", getURL(dbms, host, port, sid));
			
			break;
		    }
		}
	    }
	}
	
	writeToXml(document);
    }
    
    private static String getDriverName(String dbms){
	if( dbms.equalsIgnoreCase("mysql") ){
	    return "com.mysql.jdbc.Driver";
	} else if( dbms.equalsIgnoreCase("oracle") ){
	    return "oracle.jdbc.driver.OracleDriver";
	} else if( dbms.equalsIgnoreCase("mssql") ){
	    return "com.microsoft.jdbc.sqlserver.SQLServerDriver";
	}
	return null;
    }
    
    private static String getURL(String dbms, String host, int port, String sid){
	if( dbms.equalsIgnoreCase("mysql") ){
	    return "jdbc:mysql://" + host + ":" + port + "/" + sid;
	} else if( dbms.equalsIgnoreCase("oracle") ){
	    return "jdbc:oracle:thin:@" + host + ":" + port + ":" + sid;
	} else if( dbms.equalsIgnoreCase("mssql") ){
	    return "jdbc:microsoft:sqlserver://" + host + ":" + port + ";DatabaseName=" + sid;
	}
	return null;
    }
}
