/*
 * DAOQueryFile.java
 *
 */

package justfun.dao.helper.util;

import java.io.File;
import java.io.FileInputStream;
import java.io.PrintWriter;
import java.io.Writer;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import org.dom4j.Document;
import org.dom4j.DocumentFactory;
import org.dom4j.Element;
import org.dom4j.io.OutputFormat;
import org.dom4j.io.SAXReader;
import org.dom4j.io.XMLWriter;

/**
 *
 * @author justfun
 */
public class QueryLoader {
    
    public static Map loadFromProperties(File file) throws Exception {
        Map data = new HashMap();
        
        Properties prop = new Properties();
        
        FileInputStream fis = new FileInputStream(file);
        try{
            prop.load(fis);
            
            String key;
            for( Iterator i = prop.keySet().iterator(); i.hasNext(); ){
                key = i.next().toString();
                data.put(key, prop.getProperty(key));
            }
        }finally{
            if( fis != null )
                fis.close();
        }
        
        return data;
    }
    
    public static void storeToProperties(File file, Map data) throws Exception {
        /*
        Properties prop = new Properties();
        if( data != null ){
            String key;
            for( Iterator i = data.keySet().iterator(); i.hasNext(); ){
                key = i.next().toString();
                prop.setProperty(key, data.get(key).toString().replaceAll("\n", "\\"));
            }
        }
         
        FileOutputStream fos = new FileOutputStream(file);
        try{
            prop.store(fos, "");
        }finally{
            if( fos != null )
                fos.close();
        }
         */
        
        if( data != null ){
            PrintWriter writer = new PrintWriter(file, "UTF-8");
            try{
                String key, value;
                for( Iterator i = data.keySet().iterator(); i.hasNext(); ){
                    key = i.next().toString();
                    value = data.get(key).toString().replaceAll("\n", "\\\\\n");
                    
                    writer.println(key + " = " + value);
                }
            }finally{
                if( writer != null ){
                    writer.close();
                }
            }
        }
    }
    
    public static Map loadFromXml(File file) throws Exception {
        Map data = new HashMap();
        
        SAXReader reader = new SAXReader();
        Document document = reader.read(file);
        List list = document.selectNodes("dao/sql");
        Element element;
        for( Iterator i = list.iterator(); i.hasNext(); ){
            element = (Element) i.next();
            data.put(element.attributeValue("id"), element.getText());
        }
        
        return data;
    }
    
    public static void storeToXml(File file, Map data) throws Exception {
        DocumentFactory factory = DocumentFactory.getInstance();
        Document document = factory.createDocument("UTF-8");
        
        Element daoEl = factory.createElement("dao");
        document.setRootElement(daoEl);
        
        if( data != null ){
            String key;
            for( Iterator i = data.keySet().iterator(); i.hasNext(); ){
                key = i.next().toString();
                Element element = factory.createElement("sql");
                element.addAttribute("id", key);
                element.setText(data.get(key).toString());
                
                daoEl.add(element);
            }
        }
        
        XMLWriter writer = null;
        try {
            OutputFormat format = OutputFormat.createPrettyPrint();
            format.setTrimText(false);
            Writer fw = new PrintWriter(file, "UTF-8");
            writer = new XMLWriter(fw, format);
            writer.write(document);
        } finally {
            if( writer != null )
                writer.close();
        }
    }
    
}
