/*
 * 유틸리티
 *
 */

package justfun.dao.helper.util;

import java.awt.Dialog;
import java.awt.Point;
import java.awt.Rectangle;
import java.sql.ResultSetMetaData;
import java.io.File;
import java.net.URL;
import java.sql.ResultSet;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Vector;
import org.dom4j.Attribute;
import org.dom4j.DocumentFactory;
import org.dom4j.Element;

/**
 *
 * @author jhcho
 */
public class Utils {
    
    /** 파일 확장자 얻기 */
    public static String getExtension(File f) {
        String ext = null;
        String s = f.getName();
        int i = s.lastIndexOf('.');
        
        if (i > 0 &&  i < s.length() - 1) {
            ext = s.substring(i+1).toLowerCase();
        }
        return ext;
    }
    
    /** 리소스 얻기 */
    public static URL getResource(String name){
        return Thread.currentThread().getContextClassLoader().getResource("justfun/dao/helper/resource/" + name);
    }
    
    
    /**
     * XML Element를 Map으로 변환
     */
    public static Map elementToMap(Element element){
        Map map = new HashMap();
        
        List attributes = element.attributes();
        for( int i = attributes.size()-1; i >= 0; i-- ){
            Attribute attr = ((Attribute) attributes.get(i));
            map.put(attr.getName(), attr.getValue());
        }
        
        return map;
    }
    
    /**
     * Map을 XML Element로 변환
     */
    public static Element mapToElement(Map map, String elementName){
        Element element = DocumentFactory.getInstance().createElement(elementName);
        Object[] keys = map.keySet().toArray();
        for( int i = 0; i < keys.length; i++ ){
            element.addAttribute(keys[i].toString(), map.get(keys[i]).toString());
        }
        return element;
    }
    
    /**
     * ResultSet을 List로 변환
     */
    public static List resultsetToList(ResultSet rs) throws Exception {
        List list = new Vector();
        
        if( rs != null ){
            ResultSetMetaData rsmd = rs.getMetaData();
            int columnCount = rsmd.getColumnCount();
            
            String[] columnNames = new String[columnCount];
            for(int i = 0; i < columnCount; i++){
                columnNames[i] = rsmd.getColumnLabel(i+1);
            }
            
            Map record;
            while( rs.next() ){
                record = new HashMap();
                for( int i = 0; i < columnCount; i++ ){
                    record.put(columnNames[i], rs.getObject(i+1));
                }
                
                list.add(record);
            }
            rs.close();
        }
        
        return list;
    }
    
    /** 쿼리에서 사용자 정의 변수 추출 */
    public static List extractUserDefinedVariables(String sql){
        List list = new Vector();
        String startTag = "&[", endTag = "]";
        String source = sql;
        int startIndex, endIndex;
        while( (startIndex = source.indexOf(startTag)) >= 0 ){
            if( (endIndex = source.indexOf(endTag)) >= 0 ){
                String key = source.substring(startIndex + startTag.length(), endIndex);
                list.add(key);
                source = source.substring(endIndex+endTag.length());
            }
        }
        
        return list;
    }
    
    /** Dialog를 화면(부모 컨테이너) 중앙으로 위치 */
    public static void setDialogLocationToCenter(Dialog dialog){
        Rectangle ownerPos = dialog.getParent().getBounds();
        Rectangle dialogPos = dialog.getBounds();
        
        double x = ownerPos.getX() + (ownerPos.getWidth() - dialogPos.getWidth()) / 2;
        double y = ownerPos.getY() + (ownerPos.getHeight() - dialogPos.getHeight()) / 2;
        
        Point dialogNewPos = new Point();
        dialogNewPos.setLocation(x, y);
        
        dialog.setLocation(dialogNewPos);
    }
}
