/*
 * Created on 2005. 10. 13.
 *
 */
package justfun.dao;

/**
 * DAO에서 처리할 작업을 표현하는 클래스
 * 
 * @author 조재호
 * 
 */
public class Task {

	public static final int SELECT = 0;
	public static final int UPDATE = 1;
	public static final int PROCEDURE = 2;
	public static final int FUNCTION = 3;

	private int type;
	private String query;
	private Object[] param;

	private Object result;

	/**
	 * 생성자. 작업을 생성한다.
	 * 
	 * @param query
	 *            쿼리
	 * @param param
	 *            바인딩 변수
	 */
	public Task(int type, String query, Object[] param) {
		if (type < SELECT || type > FUNCTION)
			throw new IllegalArgumentException("Invalid Task Type!!");

		this.type = type;
		this.query = query;
		this.param = param;
	}

	/**
	 * @return Returns the type.
	 */
	public int getType() {
		return type;
	}

	/**
	 * @return Returns the param.
	 */
	public Object[] getParam() {
		return param;
	}

	/**
	 * @return Returns the query.
	 */
	public String getQuery() {
		return query;
	}

	/**
	 * @return Returns the result.
	 */
	public Object getResult() {
		return result;
	}

	/**
	 * DAOWorker에 의해 Task가 수행될 때 DAOWorker가 Task의 결과를 넘겨주기 위한 용도로 사용한다.
	 * 
	 * @param result
	 *            The result to set.
	 */
	protected void setResult(Object result) {
		this.result = result;
	}
}
