/*
 * Created on 2005. 11. 24.
 *
 */
package justfun.dao;

import java.util.HashMap;
import java.util.Map;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.DOMException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

public class XmlQueryLoader {

	private XmlQueryLoader() {

	}

	private static class Holder {
		static final XmlQueryLoader INSTANCE = new XmlQueryLoader();
	}

	/**
	 * XmlQueryLoader 객체를 얻는다.
	 * 
	 * @return XmlQueryLoader의 Singleton 객체
	 */
	public static XmlQueryLoader instance() {
		return Holder.INSTANCE;
	}

	/**
	 * 쿼리 로딩
	 * 
	 * @param file
	 *            ClassLoader path
	 * @return
	 */
	public Map load(String file) throws Exception {
		Map map = new HashMap();

		if (file.startsWith("/"))
			file = file.substring(1);

		ClassLoader loader = Thread.currentThread().getContextClassLoader();
		if (loader == null)
			loader = getClass().getClassLoader();

		DocumentBuilder builder = DocumentBuilderFactory.newInstance().newDocumentBuilder();
		Document document = builder.parse(loader.getResourceAsStream(file));

		// Root Element 를 얻는다.
		Element rootElement = document.getDocumentElement();

		if (!rootElement.getNodeName().equals("tinydao") && !rootElement.getNodeName().equals("dao")) {
			throw new DOMException(DOMException.VALIDATION_ERR, "Invalid Root Element: " + rootElement.getNodeName());
		}

		if (rootElement.getNodeName().equals("tinydao")) {
			System.out.println("[DAOManager] !!WARN!! Detect deprecated root element: 'tinydao'. (Recommended root element is 'dao') ");
		}
		NodeList nodeList = rootElement.getElementsByTagName("sql");

		String id, query;
		if (nodeList != null) {
			for (int i = 0; i < nodeList.getLength(); i++) {
				id = nodeList.item(i).getAttributes().getNamedItem("id").getNodeValue();
				query = nodeList.item(i).getChildNodes().item(0).getNodeValue();

				map.put(id, query);
			}
		}

		return map;
	}
}
