package justfun.framework.svc.config;

import java.io.File;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

/**
 * <code>Properties</code>를 사용하는 ServiceConfig
 * 
 * @author 조재호
 * 
 */
public class PropertiesServiceConfig implements ServiceConfig {

	protected ResourceBundle resource;

	protected String configName;
	protected File configFile;
	protected long configTime;

	public PropertiesServiceConfig() {
	}

	/**
	 * Properties 파일로부터 환경 설정 정보를 로딩한다.
	 * 
	 * @param configName
	 *            ResourceBundle의 name 규약을 따른다.
	 */
	public void load(String configName) throws ServiceConfigException {
		/* 설정 파일 LOAD */
		this.resource = ResourceBundle.getBundle(configName);
		this.configName = configName;

		ClassLoader loader = Thread.currentThread().getContextClassLoader();
		if (loader == null)
			loader = getClass().getClassLoader();

		this.configFile = new File(loader.getResource(configName.replace(".", File.separator) + ".properties").toString());
		this.configTime = this.configFile.lastModified();
	}

	public void reload() throws ServiceConfigException {
		load(this.configName);
	}

	public long getConfigTime() {
		return configTime;
	}

	public long getLastModified() {
		return configFile.lastModified();
	}

	public boolean isModified() {
		return (configTime != configFile.lastModified());
	}

	public List getKeys() {
		List list = new ArrayList();
		for (Enumeration e = resource.getKeys(); e.hasMoreElements();) {
			list.add(e.nextElement());
		}
		return list;
	}

	public boolean containsKey(String key) {
		try {
			return (resource.getString(key) != null);
		}
		catch (MissingResourceException mre) {
			return false;
		}
	}

	public String getServiceName(String serviceID) {
		return resource.getString("service." + serviceID);
	}

	public String getServiceType(String serviceID) {
		return resource.getString("service." + serviceID + ".type");
	}

	public String getValue(String key) {
		return resource.getString(key);
	}

	public String getValue(String key, String defaultValue) {
		String returnValue = resource.getString(key);
		return (returnValue == null) ? defaultValue : returnValue;
	}

}
