package justfun.framework.svc.controller;

import justfun.framework.svc.factory.ServiceFactory;
import justfun.framework.svc.factory.ServiceFactoryManager;
import justfun.framework.svc.service.Service;
import justfun.framework.svc.service.ServiceParameter;

/**
 * <p>
 * 서비스 프로세서 구현
 * </p>
 * 
 * <code><pre>
 * public ServiceParameter process(String factoryConfFile, String serviceID, ServiceParameter param)
 * 		throws Exception {
 * 	ServiceFactoryManager sfm = ServiceFactoryManager.getInstance();
 * 	ServiceFactory factory = sfm.findFactory(factoryConfFile);
 * 	Service service = factory.findService(serviceID);
 * 	ServiceParameter returnVal = service.execute(param);
 * 
 * 	return returnVal;
 * }
 * </pre></code>
 * 
 * @author 조재호
 */
public class ServiceProcessor {

	/**
	 * 전처리 수행
	 * 
	 */
	public void preProcess() {
	}

	/**
	 * 후처리 수행
	 * 
	 */
	public void postProcess() {
	}

	/**
	 * 프로세서 수행.
	 * 
	 * @param configName
	 * @param serviceID
	 * @param param
	 * @return 서비스 수행 결과를 리턴
	 * @throws Exception
	 */
	public final ServiceParameter process(String configName, String serviceID, ServiceParameter param)
			throws Exception {

		preProcess();

		ServiceFactoryManager sfm = ServiceFactoryManager.getInstance();
		ServiceFactory factory = sfm.findFactory(configName);
		Service service = factory.findService(serviceID);
		ServiceParameter returnVal = service.execute(param);

		postProcess();

		return returnVal;
	}

}
