package justfun.framework.svc.factories;

import java.lang.reflect.Constructor;

import justfun.framework.svc.config.ServiceConfig;
import justfun.framework.svc.factory.AbstractServiceFactory;
import justfun.framework.svc.factory.ServiceFactoryException;
import justfun.framework.svc.factory.ServiceFactoryManager;
import justfun.framework.svc.service.Service;

/**
 * 서비스 팩토리 구현
 * 
 * @author 조재호
 */
public class ServiceFactory extends AbstractServiceFactory {

	/* ServiceFactoryManager */
	protected ServiceFactoryManager factoryManager = ServiceFactoryManager.getInstance();

	/* for MessageResourceBundle */
	protected Class clazz = this.getClass();

	/* for Service's Constructor */
	protected Class[] type = new Class[] { String.class, String.class, ServiceConfig.class };

	/**
	 * @param configName
	 * @param config
	 */
	public ServiceFactory(String configName, ServiceConfig config) throws ServiceFactoryException {
		super(configName, config);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see justfun.framework.svc.factory.ServiceFactoryIF#findService(java.lang.String)
	 */
	public Service findService(String serviceID) throws ServiceFactoryException {

		/* Service가 없으면 Exception 발생 */
		if (!config.containsKey("service." + serviceID))
			throw new ServiceFactoryException("Can't find Service : key service." + serviceID);

		/* Service Type 설정 값을 얻는다. */
		String svcType = config.getServiceType(serviceID);
		if (svcType == null)
			throw new ServiceFactoryException("Can'y find Service Type : key service." + serviceID + ".type");

		/* Service 이름을 얻는다. */
		String svcName = config.getServiceName(serviceID);

		try {
			Class clazz = Class.forName(svcType);
			Constructor constructor = clazz.getConstructor(type);
			Object obj = constructor.newInstance(new Object[] { serviceID, svcName, config });
			Service service = (Service) obj;

			return service;
		}
		catch (ClassCastException cce) {
			factoryManager.removeFactory(configName);

			throw new ServiceFactoryException("Invalid Service Type : " + serviceID, cce);
		}
		catch (Exception ex) {
			factoryManager.removeFactory(configName);

			throw new ServiceFactoryException(ex);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see justfun.framework.svc.factory.ServiceFactoryIF#getServices()
	 */
	/*
	 * public Map getServices() { return config.getServices(); }
	 */

	/*
	 * (non-Javadoc)
	 * 
	 * @see justfun.framework.svc.factory.ServiceFactory#getConfig()
	 */
	public ServiceConfig getConfig() {
		return config;
	}

}
