package justfun.framework.svc.factory;

import justfun.framework.svc.config.ServiceConfig;
import justfun.framework.svc.service.Service;

/**
 * 서비스 팩토리 인터페이스 정의
 * 
 * @author 조재호
 */

public interface ServiceFactory {

	/**
	 * 해당 ID의 서비스 객체를 리턴한다.
	 * 
	 * @param serviceID
	 * @return serviceID에 해당하는 서비스 객체
	 * @throws serviceID에
	 *             해당하는 서비스 객체를 찾지 못하거나, 서비스 객체 생성에 실패할 경우
	 *             ServiceFactoryException이 발생한다.
	 */
	public Service findService(String serviceID) throws ServiceFactoryException;

	/**
	 * ServiceFactory의 환경 설정 정보를 가져온다.
	 * 
	 * @return ȯ�� 환경 설정 정보
	 */
	public ServiceConfig getConfig();

}
