package justfun.framework.svc.services;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.Map;

import justfun.framework.svc.config.ServiceConfig;
import justfun.framework.svc.service.AbstractService;
import justfun.framework.svc.service.ServiceException;
import justfun.framework.svc.service.ServiceParameter;

/**
 * 서비스 클래스 구현. 서비스 클래스 하나에 한 개 이상의 비즈니스 로직을 구현할 수 있다.
 * 
 * 비즈니스 로직을 처리하는 각각의 메소드를 만들고, execute 메소드 호출 시 input data의 invoke.method에 호출할
 * 메소드 이름을 넘겨준다.
 * 
 * @author 조재호
 */
public class AdvancedService extends AbstractService {

	protected final Class[] type = new Class[] { ServiceParameter.class };
	protected Class clazz = this.getClass();

	protected Map methods = new HashMap();

	/**
	 * @param serviceID
	 * @param serviceName
	 */
	public AdvancedService(String serviceID, String serviceName, ServiceConfig config) {
		super(serviceID, serviceName, config);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see kr.co.cybercell.framework.service.Service#execute(java.lang.Object)
	 */
	public ServiceParameter execute(ServiceParameter param) throws ServiceException {
		if (!config.containsKey("service." + serviceID + ".invoke.key"))
			throw new ServiceException("invoke.key was not defined : " + serviceID);

		String invokeMethod = (String) param.get(config.getValue("service." + serviceID + ".invoke.key"));
		if (invokeMethod == null)
			throw new ServiceException("Can't find invoke key : key service." + serviceID + ".invoke.key");

		ServiceParameter output = null;
		try {
			Method method = getMethod(invokeMethod);
			output = (ServiceParameter) method.invoke(this, new Object[] { param });
		}
		catch (NoSuchMethodException nsme) {
			throw new ServiceException(nsme);
		}
		catch (IllegalAccessException iae) {
			throw new ServiceException(iae);
		}
		catch (InvocationTargetException ite) {
			throw new ServiceException(ite);
		}
		catch (ClassCastException cce) {
			throw new ServiceException("method \"" + invokeMethod + "\" must return java.util.Map", cce);
		}

		return output;
	}

	/**
	 * 
	 * 
	 * @param name
	 * @return
	 * @throws NoSuchMethodException
	 */
	protected Method getMethod(String name) throws NoSuchMethodException {
		synchronized (methods) {
			Method method = (Method) methods.get(name);
			if (method == null) {
				method = clazz.getMethod(name, type);
				methods.put(name, method);
			}
			return (method);
		}

	}

}
